@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -24,18 +24,13 @@
  * Some additions by: Dave Mielke <dave@mielke.cc>
  */
 
-#define VERSION \
-"Braille Lite driver, version 0.6.0 (June 2003)"
-
 #include "prologue.h"
 
-#include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 #include <string.h>
 
-#include "Programs/misc.h"
-#include "Programs/message.h"
+#include "misc.h"
+#include "message.h"
 
 typedef enum {
   PARM_BAUDRATE,
@@ -43,10 +38,10 @@ typedef enum {
 } DriverParameter;
 #define BRLPARMS "baudrate", "kbemu"
 
-#include "Programs/brl_driver.h"
-#include "Programs/tbl.h"
+#include "brl_driver.h"
+#include "tbl.h"
 #include "braille.h"
-#include "Programs/io_serial.h"
+#include "io_serial.h"
 
 #define BL_NEED_ARRAYS
 #include "bindings.h"		/* for keybindings */
@@ -261,17 +256,6 @@ qflush (void)
   qlen = 0;
 }
 
-
-static void
-brl_identify (void)
-{
-  LogPrint(LOG_NOTICE, VERSION);
-  /* I don't mean to take away anyone's copyright, but now that significant
-     modifs have been made in more recent years by 2-3 others... it'd become
-     bulky IMHO so let's just live without the copyright notice. */
-  /*LogPrint(LOG_INFO, "   Copyright (C) 1998 by Nikhil Nair.");*/
-}
-
 static int
 await_ack (void) {
   hasTimedOut(0);
@@ -291,7 +275,7 @@ write_prebrl (void) {
 }
 
 static int
-brl_open (BrailleDisplay *brl, char **parameters, const char *device)
+brl_construct (BrailleDisplay *brl, char **parameters, const char *device)
 {
   static const int good_baudrates[] =
     {300,600,1200,2400,4800,9600,19200,38400, 0};
@@ -304,8 +288,8 @@ brl_open (BrailleDisplay *brl, char **parameters, const char *device)
     baudrate = BAUDRATE;
 
   if (*parameters[PARM_KBEMU])
-    validateYesNo(&kbemu, "keyboard emulation initial state",
-		  parameters[PARM_KBEMU]);
+    if (!validateYesNo(&kbemu, parameters[PARM_KBEMU]))
+      LogPrint(LOG_WARNING, "%s: %s", "invalid keyboard emulation setting", parameters[PARM_KBEMU]);
   kbemu = !!kbemu;
 
   {
@@ -416,7 +400,7 @@ brl_open (BrailleDisplay *brl, char **parameters, const char *device)
 
 
 static void
-brl_close (BrailleDisplay * brl)
+brl_destruct (BrailleDisplay * brl)
 {
   if (rawdata) {
     free(rawdata);
@@ -440,14 +424,8 @@ brl_close (BrailleDisplay * brl)
 }
 
 
-static void
-brl_writeStatus (BrailleDisplay *brl, const unsigned char *s)
-{
-}
-
-
-static void
-brl_writeWindow (BrailleDisplay * brl)
+static int
+brl_writeWindow (BrailleDisplay * brl, const wchar_t *text)
 {
   short i;			/* loop counter */
 
@@ -456,7 +434,7 @@ brl_writeWindow (BrailleDisplay * brl)
    * This is highly antisocial behaviour!
    */
   if (intoverride)
-    return;
+    return 1;
 
   /* First, the internal cursor: */
   if (int_cursor)
@@ -495,12 +473,13 @@ brl_writeWindow (BrailleDisplay * brl)
       waiting_ack = 1;
       /* send the ^ED... */
       write_prebrl();
-      if (!await_ack()) return;
+      if (!await_ack()) return 1;
 
       /* OK, now we'll suppose we're all clear to send Braille data. */
       serialWriteData(BL_serialDevice, rawdata, blitesz);
       await_ack();
     }
+  return 1;
 }
 
 
