@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -34,6 +34,7 @@
 
 #include "misc.h"
 
+#define BRL_HAVE_STATUS_CELLS
 #include "brl_driver.h"
 #include "braille.h"
 #include "io_serial.h"
@@ -266,7 +267,7 @@ static void brl_destruct(BrailleDisplay *brl)
 }
 
 
-static void brl_writeWindow(BrailleDisplay *brl)
+static int brl_writeWindow(BrailleDisplay *brl, const wchar_t *text)
 {
   int i, j;
 
@@ -284,13 +285,15 @@ static void brl_writeWindow(BrailleDisplay *brl)
      
   /* Write to Braille Display */
   WriteToBrlDisplay(rawdata);
+  return 1;
 }
 
 
-static void brl_writeStatus(BrailleDisplay *brl, const unsigned char *st)
+static int brl_writeStatus(BrailleDisplay *brl, const unsigned char *st)
 {
   /* Update status cells */
   memcpy(Status, st, model->NbStCells);
+  return 1;
 }
 
 
