@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -35,6 +35,7 @@ typedef enum {
 #define BRLPARMS "debugpackets", "statuscells"
 
 #define BRLSTAT ST_AlvaStyle
+#define BRL_HAVE_STATUS_CELLS
 #define BRL_HAVE_PACKET_IO
 #define BRL_HAVE_FIRMNESS
 #include "brl_driver.h"
@@ -832,15 +833,17 @@ brl_destruct (BrailleDisplay *brl) {
   io->closePort();
 }
 
-static void
-brl_writeWindow (BrailleDisplay *brl) {
+static int
+brl_writeWindow (BrailleDisplay *brl, const wchar_t *text) {
   updateCells(brl, brl->buffer, textCells, textOffset);
   writeRequest(brl);
+  return 1;
 }
 
-static void
+static int
 brl_writeStatus (BrailleDisplay *brl, const unsigned char *status) {
   updateCells(brl, status, statusCells, statusOffset);
+  return 1;
 }
 
 static int
@@ -1129,6 +1132,13 @@ brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
                   case 0:
                     command = BRL_BLK_ROUTE;
                     break;
+                  case (KEY_WHEEL_LEFT):
+                    command = BRL_BLK_SETLEFT;
+                    break;
+                  case (KEY_WHEEL_RIGHT):
+                    command = BRL_BLK_DESCCHAR;
+                    break;
+
                   case (KEY_ADVANCE_LEFT):
                     command = BRL_BLK_CUTBEGIN;
                     break;
@@ -1318,6 +1328,6 @@ brl_reset (BrailleDisplay *brl) {
 
 static void
 brl_firmness (BrailleDisplay *brl, BrailleFirmness setting) {
-  firmnessSetting = setting * 0XFF / BF_MAXIMUM;
+  firmnessSetting = setting * 0XFF / BRL_FIRMNESS_MAXIMUM;
   writeRequest(brl);
 }
