@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -29,6 +29,7 @@ typedef enum {
 #define BRLPARMS "inputmode"
 
 #define BRLSTAT ST_AlvaStyle
+#define BRL_HAVE_STATUS_CELLS
 #define BRL_HAVE_PACKET_IO
 #define BRL_HAVE_SENSITIVITY
 #include "brl_driver.h"
@@ -728,8 +729,8 @@ updateCells (BrailleDisplay *brl) {
   return 1;
 }
 
-static void
-brl_writeWindow (BrailleDisplay *brl) {
+static int
+brl_writeWindow (BrailleDisplay *brl, const wchar_t *text) {
   if (memcmp(brl->buffer, prevData, model->textCells) != 0) {
     int i;
     for (i=0; i<model->textCells; ++i) {
@@ -738,9 +739,10 @@ brl_writeWindow (BrailleDisplay *brl) {
     updateRequired = 1;
   }
   updateCells(brl);
+  return 1;
 }
 
-static void
+static int
 brl_writeStatus (BrailleDisplay *brl, const unsigned char *st) {
   if (model->statusCells &&
       (memcmp(st, prevStatus, model->statusCells) != 0)) {
@@ -750,6 +752,7 @@ brl_writeStatus (BrailleDisplay *brl, const unsigned char *st) {
     }
     updateRequired = 1;
   }
+  return 1;
 }
 
 static int
@@ -1715,7 +1718,7 @@ brl_writePacket (BrailleDisplay *brl, const void *packet, size_t length) {
 static void
 brl_sensitivity (BrailleDisplay *brl, BrailleSensitivity setting) {
   if (model->hasATC) {
-    unsigned char value = 0XFF - (setting * 0XF0 / BS_MAXIMUM);
+    unsigned char value = 0XFF - (setting * 0XF0 / BRL_SENSITIVITY_MAXIMUM);
     setAtcSensitivity(brl, value);
   }
 }
