@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -21,7 +21,7 @@
 
 #include "misc.h"
 
-//#define BRL_HAVE_VISUAL_DISPLAY
+//#define BRL_HAVE_STATUS_CELLS
 //#define BRL_HAVE_PACKET_IO
 //#define BRL_HAVE_KEY_CODES
 //#define BRL_HAVE_FIRMNESS
@@ -73,19 +73,17 @@ brl_keyToCommand (BrailleDisplay *brl, BRL_DriverCommandContext context, int key
 }
 #endif /* BRL_HAVE_KEY_CODES */
 
-static void
-brl_writeWindow (BrailleDisplay *brl) {
+static int
+brl_writeWindow (BrailleDisplay *brl, const wchar_t *text) {
+  return 1;
 }
 
-static void
+#ifdef BRL_HAVE_STATUS_CELLS
+static int
 brl_writeStatus (BrailleDisplay *brl, const unsigned char *status) {
+  return 1;
 }
-
-#ifdef BRL_HAVE_VISUAL_DISPLAY
-static void
-brl_writeVisual (BrailleDisplay *brl) {
-}
-#endif /* BRL_HAVE_VISUAL_DISPLAY */
+#endif /* BRL_HAVE_STATUS_CELLS */
 
 static int
 brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
