@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,9 +15,6 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#define TS_VERSION "BRLTTY driver for TSI displays, version 2.74 (April 2004)"
-#define TS_COPYRIGHT "Copyright (C) 1996-2004 by Stphane Doyon " \
-                  "<s.doyon@videotron.ca>"
 /* TSI/braille.c - Braille display driver for TSI displays
  *
  * Written by Stphane Doyon (s.doyon@videotron.ca)
@@ -81,18 +78,17 @@
 
 #include "prologue.h"
 
-#include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 #include <string.h>
 
-#include "Programs/misc.h"
-#include "Programs/message.h"
+#include "misc.h"
+#include "message.h"
 
 #define BRLSTAT ST_PB80Style
-#include "Programs/brl_driver.h"
+#define BRL_HAVE_STATUS_CELLS
+#include "brl_driver.h"
 #include "braille.h"
-#include "Programs/io_serial.h"
+#include "io_serial.h"
 
 /* Braille display parameters that do not change */
 #define BRLROWS 1		/* only one row on braille display */
@@ -144,22 +140,22 @@ static TranslationTable outputTable;
 #define BAUD_DELAY (100)
 
 /* Communication codes */
-static char BRL_QUERY[] = {0xFF, 0xFF, 0x0A};
+static unsigned char BRL_QUERY[] = {0xFF, 0xFF, 0x0A};
 #define DIM_BRL_QUERY 3
-static char BRL_TYPEMATIC[] = {0xFF, 0xFF, 0x0D};
+static unsigned char BRL_TYPEMATIC[] = {0xFF, 0xFF, 0x0D};
 #define DIM_BRL_TYPEMATIC 3
 #ifdef HIGHBAUD
 /* Command to put the PB at 19200baud */
-static char BRL_UART192[] = {0xFF, 0xFF, 0x05, 0x04};
+static unsigned char BRL_UART192[] = {0xFF, 0xFF, 0x05, 0x04};
 #define DIM_BRL_UART192 4
 #endif /* HIGHBAUD */
 #if 0
 /* Activate handshake ? */
-static char BRL_UART_HANDSHAK[] = {0xFF, 0xFF, 0x05, 0x01};
+static unsigned char BRL_UART_HANDSHAK[] = {0xFF, 0xFF, 0x05, 0x01};
 #define DIM_BRL_UART_HANDSHAK 4
 #endif /* 0 */
 /* Normal header for sending dots, with cursor always off */
-static char BRL_SEND_HEAD[] = {0xFF, 0xFF, 0x04, 0x00, 0x00, 0x01};
+static unsigned char BRL_SEND_HEAD[] = {0xFF, 0xFF, 0x04, 0x00, 0x00, 0x01};
 #define DIM_BRL_SEND_FIXED 6
 #define DIM_BRL_SEND 8
 /* Two extra bytes for lenght and offset */
@@ -302,14 +298,6 @@ static char disp_ver[Q_VER_LENGTH]; /* version of the hardware */
 
 static enum { NAV20_40=0, NAV80, PB40, PB65_80} displayType;
 
-static void 
-brl_identify (void)
-{
-  LogPrint(LOG_NOTICE, TS_VERSION);
-  LogPrint(LOG_INFO, "   "TS_COPYRIGHT);
-}
-
-
 static int
 myread(void *buf, unsigned len)
 {
@@ -329,7 +317,7 @@ QueryDisplay(unsigned char *reply)
           LogPrint(LOG_DEBUG, "Valid reply received.");
           return 1;
         } else {
-          LogBytes("Unexpected response", reply, count);
+          LogBytes(LOG_WARNING, "Unexpected Response", reply, count);
         }
       } else {
         LogError("Read");
@@ -356,7 +344,7 @@ ResetTypematic (void)
 
 
 static int
-brl_open (BrailleDisplay *brl, char **parameters, const char *device)
+brl_construct (BrailleDisplay *brl, char **parameters, const char *device)
 {
   int i=0;
   unsigned char reply[Q_REPLY_LENGTH];
@@ -530,13 +518,13 @@ brl_open (BrailleDisplay *brl, char **parameters, const char *device)
   return 1;
 
 failure:;
-  brl_close(brl);
+  brl_destruct(brl);
   return 0;
 }
 
 
 static void 
-brl_close (BrailleDisplay *brl)
+brl_destruct (BrailleDisplay *brl)
 {
   if (serialDevice) {
     serialCloseDevice(serialDevice);
@@ -638,7 +626,7 @@ display (const unsigned char *pattern,
   };
 }
 
-static void
+static int
 brl_writeStatus (BrailleDisplay *brl, const unsigned char *s)
 /* Only the PB80, which actually has 81cells, can be considered to have status
    cells, and it has only one. We could also decide to devote some of
@@ -646,6 +634,7 @@ brl_writeStatus (BrailleDisplay *brl, const unsigned char *s)
 {
   if(ncells == 81)
     dispbuf[80] = s[0];
+  return 1;
 }
 
 
@@ -656,8 +645,8 @@ display_all (unsigned char *pattern)
 }
 
 
-static void 
-brl_writeWindow (BrailleDisplay *brl)
+static int 
+brl_writeWindow (BrailleDisplay *brl, const wchar_t *text)
 {
   static int count = 0;
 
@@ -676,7 +665,7 @@ brl_writeWindow (BrailleDisplay *brl)
     
     for(start=0; start<ncells; start++)
       if(dispbuf[start] != prevdata[start]) break;
-    if(start == ncells) return;
+    if(start == ncells) return 1;
     for(stop = ncells-1; stop > start; stop--)
       if(dispbuf[stop] != prevdata[stop]) break;
     
@@ -711,6 +700,7 @@ brl_writeWindow (BrailleDisplay *brl)
     if (collecting)
       display (dispbuf, base, i - simil - 1);
   }
+return 1;
 }
 
 
