@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -41,7 +41,7 @@ static iconv_t conversionDescriptor = NULL;
 #include <ncurses.h>
 #elif defined(HAVE_PKG_NCURSESW)
 #define USE_CURSES
-#include <ncurses.h>
+#include <ncursesw/ncurses.h>
 #else /* HAVE_PKG_ */
 #warning curses package either unspecified or unsupported
 #define addstr(string) serialWriteData(ttyDevice, string, strlen(string))
@@ -50,6 +50,7 @@ static iconv_t conversionDescriptor = NULL;
 #endif /* HAVE_PKG_CURSES */
 
 #include "misc.h"
+#include "charset.h"
 
 #ifdef USE_CURSES
 #define BRLPARM_TERM "term",
@@ -82,7 +83,6 @@ typedef enum {
 #define BRLPARMS "baud", BRLPARM_TERM "lines", "columns", BRLPARM_CHARSET "locale"
 
 #define BRL_HAVE_KEY_CODES
-#define BRL_HAVE_VISUAL_DISPLAY
 #include "brl_driver.h"
 #include "braille.h"
 #include "io_serial.h"
@@ -114,7 +114,7 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
   int windowColumns = 40;
 
 #ifdef HAVE_ICONV_H
-  char *characterSet = "ISO8859-1";
+  const char *characterSet = getLocaleCharset();
 #endif /* HAVE_ICONV_H */
 
   if (!isSerialDevice(&device)) {
@@ -164,7 +164,7 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
     classificationLocale = parameters[PARM_LOCALE];
 
 #ifdef HAVE_ICONV_H
-  if ((conversionDescriptor = iconv_open(characterSet, "ISO8859-1")) != (iconv_t)-1) {
+  if ((conversionDescriptor = iconv_open(characterSet, "WCHAR_T")) != (iconv_t)-1) {
 #endif /* HAVE_ICONV_H */
     if ((ttyDevice = serialOpenDevice(device))) {
       if (serialRestartDevice(ttyDevice, ttyBaud)) {
@@ -239,17 +239,13 @@ brl_destruct (BrailleDisplay *brl) {
 }
 
 static void
-brl_writeWindow (BrailleDisplay *brl) {
-}
-
-static void
-writeText (const unsigned char *buffer, int columns) {
+writeText (const wchar_t *buffer, int columns) {
   int column;
   for (column=0; column<columns; column++) {
-    char c = buffer[column];
+    wchar_t c = buffer[column];
 #ifdef HAVE_ICONV_H
-    char *pc = &c;
-    size_t sc = 1;
+    char *pc = (char*) &c;
+    size_t sc = sizeof(wchar_t);
     char d[MB_MAX_LEN+1];
     char *pd = d;
     size_t sd = MB_MAX_LEN;
@@ -262,14 +258,15 @@ writeText (const unsigned char *buffer, int columns) {
   }
 }
 
-static void
-brl_writeVisual (BrailleDisplay *brl) {
+static int
+brl_writeWindow (BrailleDisplay *brl, const wchar_t *text) {
   static unsigned char previousContent[MAX_WINDOW_SIZE];
   static int previousCursor = -1;
   char *previousLocale;
 
-  if (memcmp(previousContent, brl->buffer, brl->x*brl->y) == 0
-      && brl->cursor == previousCursor) return;
+  if ((memcmp(previousContent, brl->buffer, brl->x*brl->y) == 0) &&
+      (brl->cursor == previousCursor))
+    return 1;
   memcpy(previousContent, brl->buffer, brl->x*brl->y);
   previousCursor = brl->cursor;
 
@@ -289,7 +286,7 @@ brl_writeVisual (BrailleDisplay *brl) {
   {
     int row;
     for (row=0; row<brl->y; row++) {
-      writeText(&brl->buffer[row*brl->x], brl->x);
+      writeText(&text[row*brl->x], brl->x);
       if (row < brl->y-1)
         addstr("\r\n");
     }
@@ -298,21 +295,20 @@ brl_writeVisual (BrailleDisplay *brl) {
 #ifdef USE_CURSES
   if ((brl->cursor >= 0) && (brl->cursor < (brl->x * brl->y)))
     move(brl->cursor/brl->x, brl->cursor%brl->x);
+  else
+    move(brl->y, 0);
   refresh();
 #else /* USE_CURSES */
   if ((brl->y == 1) && (brl->cursor >= 0) && (brl->cursor < brl->x)) {
     addch('\r');
-    writeText(brl->buffer, brl->cursor);
+    writeText(text, brl->cursor);
   } else {
     addstr("\r\n");
   }
 #endif /* USE_CURSES */
 
   if (previousLocale) setlocale(LC_CTYPE, previousLocale);
-}
-
-static void
-brl_writeStatus(BrailleDisplay *brl, const unsigned char *cells) {
+  return 1;
 }
 
 int
