@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -55,6 +55,7 @@ typedef enum {
 #define BRLPARMS "inputmode", "statuscells"
 
 #define BRLSTAT ST_VoyagerStyle
+#define BRL_HAVE_STATUS_CELLS
 #define BRL_HAVE_FIRMNESS
 #include "brl_driver.h"
 #include "tbl.h"
@@ -381,7 +382,7 @@ openUsbPort (char **parameters, const char *device) {
   static const UsbChannelDefinition definitions[] = {
     { .vendor=0X0798, .product=0X0001, 
       .configuration=1, .interface=0, .alternative=0,
-      .inputEndpoint=1, .outputEndpoint=0
+      .inputEndpoint=1
     }
     ,
     { .vendor=0 }
@@ -669,19 +670,20 @@ brl_destruct (BrailleDisplay *brl) {
   }
 }
 
-static void
+static int
 brl_writeStatus (BrailleDisplay *brl, const unsigned char *cells) {
   memcpy(currentCells+statusOffset, cells, statusCells);
+  return 1;
 }
 
-static void
-brl_writeWindow (BrailleDisplay *brl) {
+static int
+brl_writeWindow (BrailleDisplay *brl, const wchar_t *text) {
   unsigned char buffer[totalCells];
 
   memcpy(currentCells+textOffset, brl->buffer, textCells);
 
   /* If content hasn't changed, do nothing. */
-  if (memcmp(previousCells, currentCells, totalCells) == 0) return;
+  if (memcmp(previousCells, currentCells, totalCells) == 0) return 1;
 
   /* remember current content */
   memcpy(previousCells, currentCells, totalCells);
@@ -714,6 +716,7 @@ brl_writeWindow (BrailleDisplay *brl) {
       io->writeBraille(buffer, 70, 2);
       break;
   }
+  return 1;
 }
 
 
@@ -1133,7 +1136,7 @@ brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
  */
 static void
 brl_firmness (BrailleDisplay *brl, BrailleFirmness setting) {
-  unsigned char voltage = 0XFF - (setting * 0XFF / BF_MAXIMUM);
+  unsigned char voltage = 0XFF - (setting * 0XFF / BRL_FIRMNESS_MAXIMUM);
   LogPrint(LOG_DEBUG, "Setting voltage: %02X", voltage);
   io->setDisplayVoltage(voltage);
 }
