@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -38,7 +38,8 @@ typedef enum {
   ST_VoyagerStyle
 } StatusCellsStyle;
 
-/* Braille information structure. */
+typedef struct BrailleDataStruct BrailleData;
+
 typedef struct {
   unsigned int x, y;			/* the dimensions of the display */
   unsigned int helpPage;			/* the page number within the help file */
@@ -51,37 +52,40 @@ typedef struct {
   const char *dataDirectory;
   unsigned touchEnabled:1;
   unsigned highlightWindow:1;
+  BrailleData *data;
 } BrailleDisplay;				/* used for writing to a braille display */
 
-extern void initializeBrailleDisplay (BrailleDisplay *);
-extern unsigned int drainBrailleOutput (BrailleDisplay *, int minimumDelay);
+extern void initializeBrailleDisplay (BrailleDisplay *brl);
+extern unsigned int drainBrailleOutput (BrailleDisplay *brl, int minimumDelay);
 extern int ensureBrailleBuffer (BrailleDisplay *brl, int infoLevel);
 
-extern void writeBrailleBuffer (BrailleDisplay *);
-extern void writeBrailleText (BrailleDisplay *, const char *, int);
-extern void writeBrailleString (BrailleDisplay *, const char *);
-extern void showBrailleString (BrailleDisplay *, const char *, unsigned int);
+extern int writeBrailleWindow (BrailleDisplay *brl, const wchar_t *text);
+extern int writeBrailleText (BrailleDisplay *brl, const char *text, size_t length);
+extern int writeBrailleString (BrailleDisplay *brl, const char *string);
+extern int showBrailleString (BrailleDisplay *brl, const char *string, unsigned int);
 
-extern void clearStatusCells (BrailleDisplay *brl);
-extern void setStatusText (BrailleDisplay *brl, const char *text);
+extern int clearStatusCells (BrailleDisplay *brl);
+extern int setStatusText (BrailleDisplay *brl, const char *text);
 
 extern int readBrailleCommand (BrailleDisplay *, BRL_DriverCommandContext);
 
 typedef enum {
-  BF_MINIMUM,
-  BF_LOW,
-  BF_MEDIUM,
-  BF_HIGH,
-  BF_MAXIMUM
+  BRL_FIRMNESS_MINIMUM,
+  BRL_FIRMNESS_LOW,
+  BRL_FIRMNESS_MEDIUM,
+  BRL_FIRMNESS_HIGH,
+  BRL_FIRMNESS_MAXIMUM
 } BrailleFirmness;
 
 typedef enum {
-  BS_MINIMUM,
-  BS_LOW,
-  BS_MEDIUM,
-  BS_HIGH,
-  BS_MAXIMUM
+  BRL_SENSITIVITY_MINIMUM,
+  BRL_SENSITIVITY_LOW,
+  BRL_SENSITIVITY_MEDIUM,
+  BRL_SENSITIVITY_HIGH,
+  BRL_SENSITIVITY_MAXIMUM
 } BrailleSensitivity;
+extern void setBrailleFirmness (BrailleDisplay *brl, BrailleFirmness setting);
+extern void setBrailleSensitivity (BrailleDisplay *brl, BrailleSensitivity setting);
 
 /* Routines provided by each braille driver.
  * These are loaded dynamically at run-time into this structure
@@ -94,23 +98,22 @@ typedef struct {
   int statusStyle;
 
   /* Routines provided by the braille driver library: */
-  int (*construct) (BrailleDisplay *, char **parameters, const char *);
-  void (*destruct) (BrailleDisplay *);
-  int (*readCommand) (BrailleDisplay *, BRL_DriverCommandContext);
-  void (*writeWindow) (BrailleDisplay *);
-  void (*writeStatus) (BrailleDisplay *brl, const unsigned char *);
+  int (*construct) (BrailleDisplay *brl, char **parameters, const char *device);
+  void (*destruct) (BrailleDisplay *brl);
+  int (*readCommand) (BrailleDisplay *brl, BRL_DriverCommandContext context);
+  int (*writeWindow) (BrailleDisplay *brl, const wchar_t *characters);
 
-  /* These require BRL_HAVE_VISUAL_DISPLAY. */
-  void (*writeVisual) (BrailleDisplay *);
+  /* These require BRL_HAVE_STATUS_CELLS. */
+  int (*writeStatus) (BrailleDisplay *brl, const unsigned char *cells);
 
   /* These require BRL_HAVE_PACKET_IO. */
-  ssize_t (*readPacket) (BrailleDisplay *, void *, size_t);
-  ssize_t (*writePacket) (BrailleDisplay *, const void *, size_t);
-  int (*reset) (BrailleDisplay *);
+  ssize_t (*readPacket) (BrailleDisplay *brl, void *buffer, size_t size);
+  ssize_t (*writePacket) (BrailleDisplay *brl, const void *packet, size_t size);
+  int (*reset) (BrailleDisplay *brl);
   
   /* These require BRL_HAVE_KEY_CODES. */
-  int (*readKey) (BrailleDisplay *);
-  int (*keyToCommand) (BrailleDisplay *, BRL_DriverCommandContext, int);
+  int (*readKey) (BrailleDisplay *brl);
+  int (*keyToCommand) (BrailleDisplay *brl, BRL_DriverCommandContext context, int key);
 
   /* These require BRL_HAVE_FIRMNESS. */
   void (*firmness) (BrailleDisplay *brl, BrailleFirmness setting);
@@ -132,7 +135,6 @@ typedef unsigned char TranslationTable[TRANSLATION_TABLE_SIZE];
 extern TranslationTable textTable;	 /* current text to braille translation table */
 extern TranslationTable untextTable;     /* current braille to text translation table */
 extern TranslationTable attributesTable; /* current attributes to braille translation table */
-extern void *contractionTable; /* current braille contraction table */
 
 extern void makeUntextTable (void);
 
@@ -152,11 +154,9 @@ extern const unsigned char portraitDigits[11];
 extern int portraitNumber (int x);
 extern int portraitFlag (int number, int on);
 
-extern void learnMode (BrailleDisplay *brl, int poll, int timeout);
+extern int learnMode (BrailleDisplay *brl, int poll, int timeout);
 
-extern void showDotPattern (unsigned char dots, unsigned char duration);
-extern void setBrailleFirmness (BrailleDisplay *brl, int setting);
-extern void setBrailleSensitivity (BrailleDisplay *brl, int setting);
+extern int showDotPattern (unsigned char dots, unsigned char duration);
 
 #ifdef __cplusplus
 }
