@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -24,38 +24,39 @@ extern "C" {
 
 /* this header file is used to create the driver structure
  * for a dynamically loadable braille display driver.
- * BRLNAME, BRLCODE, BRLCOMMENT, and BRLHELP must be defined - see driver make file
  */
 
 #include "brl.h"
 
 /* Routines provided by this braille display driver. */
-static void brl_identify (void);
-static int brl_open (BrailleDisplay *, char **parameters, const char *);
-static void brl_close (BrailleDisplay *);
-static int brl_readCommand (BrailleDisplay *, BRL_DriverCommandContext);
-static void brl_writeWindow (BrailleDisplay *);
-static void brl_writeStatus (BrailleDisplay *brl, const unsigned char *);
+static int brl_construct (BrailleDisplay *brl, char **parameters, const char *device);
+static void brl_destruct (BrailleDisplay *brl);
+static int brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context);
+static int brl_writeWindow (BrailleDisplay *brl, const wchar_t *characters);
 
-#ifdef BRL_HAVE_VISUAL_DISPLAY
-  static void brl_writeVisual (BrailleDisplay *);
-#endif /* BRL_HAVE_VISUAL_DISPLAY */
+#ifdef BRL_HAVE_STATUS_CELLS
+  static int brl_writeStatus (BrailleDisplay *brl, const unsigned char *cells);
+#endif /* BRL_HAVE_STATUS_CELLS */
 
 #ifdef BRL_HAVE_PACKET_IO
-  static ssize_t brl_readPacket (BrailleDisplay *, unsigned char *, size_t);
-  static ssize_t brl_writePacket (BrailleDisplay *, const unsigned char *, size_t);
-  static int brl_reset (BrailleDisplay *);
+  static ssize_t brl_readPacket (BrailleDisplay *brl, void *buffer, size_t size);
+  static ssize_t brl_writePacket (BrailleDisplay *brl, const void *buffer, size_t size);
+  static int brl_reset (BrailleDisplay *brl);
 #endif /* BRL_HAVE_PACKET_IO */
 
 #ifdef BRL_HAVE_KEY_CODES
-  static int brl_readKey (BrailleDisplay *);
-  static int brl_keyToCommand (BrailleDisplay *, BRL_DriverCommandContext, int);
+  static int brl_readKey (BrailleDisplay *brl);
+  static int brl_keyToCommand (BrailleDisplay *brl, BRL_DriverCommandContext context, int key);
 #endif /* BRL_HAVE_KEY_CODES */
 
 #ifdef BRL_HAVE_FIRMNESS
   static void brl_firmness (BrailleDisplay *brl, BrailleFirmness setting);
 #endif /* BRL_HAVE_FIRMNESS */
 
+#ifdef BRL_HAVE_SENSITIVITY
+  static void brl_sensitivity (BrailleDisplay *brl, BrailleSensitivity setting);
+#endif /* BRL_HAVE_SENSITIVITY */
+
 #ifdef BRLPARMS
   static const char *const brl_parameters[] = {BRLPARMS, NULL};
 #endif /* BRLPARMS */
@@ -65,7 +66,7 @@ static void brl_writeStatus (BrailleDisplay *brl, const unsigned char *);
 #endif /* BRLSTAT */
 
 #ifndef BRLSYMBOL
-#  define BRLSYMBOL CONCATENATE(brl_driver_,BRLCODE)
+#  define BRLSYMBOL CONCATENATE(brl_driver_,DRIVER_CODE)
 #endif /* BRLSYMBOL */
 
 #ifndef BRLCONST
@@ -74,11 +75,7 @@ static void brl_writeStatus (BrailleDisplay *brl, const unsigned char *);
 
 extern BRLCONST BrailleDriver BRLSYMBOL;
 BRLCONST BrailleDriver BRLSYMBOL = {
-  STRINGIFY(BRLNAME),
-  STRINGIFY(BRLCODE),
-  BRLCOMMENT,
-  __DATE__,
-  __TIME__,
+  DRIVER_DEFINITION_INITIALIZER,
 
 #ifdef BRLPARMS
   brl_parameters,
@@ -89,18 +86,16 @@ BRLCONST BrailleDriver BRLSYMBOL = {
   BRLHELP,
   BRLSTAT,
 
-  brl_identify,
-  brl_open,
-  brl_close,
+  brl_construct,
+  brl_destruct,
   brl_readCommand,
   brl_writeWindow,
-  brl_writeStatus,
 
-#ifdef BRL_HAVE_VISUAL_DISPLAY
-  brl_writeVisual,
-#else /* BRL_HAVE_VISUAL_DISPLAY */
-  NULL, /* brl_writeVisual */
-#endif /* BRL_HAVE_VISUAL_DISPLAY */
+#ifdef BRL_HAVE_STATUS_CELLS
+  brl_writeStatus,
+#else /* BRL_HAVE_STATUS_CELLS */
+  NULL, /* brl_writeStatus */
+#endif /* BRL_HAVE_STATUS_CELLS */
 
 #ifdef BRL_HAVE_PACKET_IO
   brl_readPacket,
@@ -121,12 +116,20 @@ BRLCONST BrailleDriver BRLSYMBOL = {
 #endif /* BRL_HAVE_KEY_CODES */
 
 #ifdef BRL_HAVE_FIRMNESS
-  brl_firmness
+  brl_firmness,
 #else /* BRL_HAVE_FIRMNESS */
-  NULL  /* brl_firmness */
+  NULL, /* brl_firmness */
 #endif /* BRL_HAVE_FIRMNESS */
+
+#ifdef BRL_HAVE_SENSITIVITY
+  brl_sensitivity
+#else /* BRL_HAVE_SENSITIVITY */
+  NULL  /* brl_sensitivity */
+#endif /* BRL_HAVE_SENSITIVITY */
 };
 
+DRIVER_VERSION_DECLARATION(brl);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
