@@ -1,7 +1,7 @@
 /*
  * libbrlapi - A library providing access to braille terminals for applications.
  *
- * Copyright (C) 2002-2007 by
+ * Copyright (C) 2002-2008 by
  *   Samuel Thibault <Samuel.Thibault@ens-lyon.org>
  *   Sbastien Hinderer <Sebastien.Hinderer@ens-lyon.org>
  *
@@ -28,7 +28,6 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <sys/stat.h>
-#include <wchar.h>
 #ifndef __MINGW32__
 #include <langinfo.h>
 #endif /* __MINGW32__ */
@@ -1131,10 +1130,6 @@ int brlapi__writeText(brlapi_handle_t *handle, int cursor, const char *str)
   char *locale;
   int res;
   size_t len;
-  if ((dispSize == 0) || (dispSize > BRLAPI_MAXPACKETSIZE/4)) {
-    brlapi_errno=BRLAPI_ERROR_INVALID_PARAMETER;
-    return -1;
-  }
   locale = setlocale(LC_CTYPE,NULL);
   wa->flags = BRLAPI_WF_REGION;
   *((uint32_t *) p) = htonl(1); p += sizeof(uint32_t);
@@ -1200,13 +1195,10 @@ endcount:
     }
     *size = htonl((p-(unsigned char *)(size+1)));
   }
-  if ((cursor>=0) && (cursor<=dispSize)) {
+  if (cursor!=BRLAPI_CURSOR_LEAVE) {
     wa->flags |= BRLAPI_WF_CURSOR;
     *((uint32_t *) p) = htonl(cursor);
     p += sizeof(uint32_t);
-  } else if (cursor!=-1) {
-    brlapi_errno = BRLAPI_ERROR_INVALID_PARAMETER;
-    return -1;
   }
 
   if ((len = getCharset(p
@@ -1805,8 +1797,8 @@ const char *brlapi_errlist[] = {
   "Authentication failed",              /* BRLAPI_ERROR_AUTHENTICATION */
 };
 
-/* brlapi_nerr: last error number */
-const int brlapi_nerr = (sizeof(brlapi_errlist)/sizeof(char*)) - 1;
+/* brlapi_nerr: number of error codes */
+const int brlapi_nerr = (sizeof(brlapi_errlist)/sizeof(char*));
 
 /* brlapi_strerror: return error message */
 const char * BRLAPI_STDCALL brlapi_strerror(const brlapi_error_t *error)
