@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -27,7 +27,6 @@
 #include <sys/stat.h>
 #include <signal.h>
 #include <time.h>
-#include <wchar.h>
 #ifdef HAVE_ICONV_H
 #include <iconv.h>
 #endif /* HAVE_ICONV_H */
@@ -61,6 +60,7 @@
 #include "brlapi_keyranges.h"
 #include "cmd.h"
 #include "brl.h"
+#include "tbl.h"
 #include "brltty.h"
 #include "misc.h"
 #include "auth.h"
@@ -98,17 +98,17 @@ const char *const api_parameters[] = { "auth", "host", "stacksize", NULL };
 static size_t stackSize;
 
 #define RELEASE "BrlAPI Server: release " BRLAPI_RELEASE
-#define COPYRIGHT "   Copyright (C) 2002-2007 by Sebastien Hinderer <Sebastien.Hinderer@ens-lyon.org>, \
+#define COPYRIGHT "   Copyright (C) 2002-2008 by Sebastien Hinderer <Sebastien.Hinderer@ens-lyon.org>, \
 Samuel Thibault <samuel.thibault@ens-lyon.org>"
 
 #define WERR(x, y, ...) do { \
-  LogPrint(LOG_DEBUG, "writing error %d", y); \
-  LogPrint(LOG_DEBUG, __VA_ARGS__); \
+  LogPrint(LOG_ERR, "writing error %d", y); \
+  LogPrint(LOG_ERR, __VA_ARGS__); \
   writeError(x, y); \
 } while(0)
 #define WEXC(x, y, type, packet, size, ...) do { \
-  LogPrint(LOG_DEBUG, "writing exception %d", y); \
-  LogPrint(LOG_DEBUG, __VA_ARGS__); \
+  LogPrint(LOG_ERR, "writing exception %d", y); \
+  LogPrint(LOG_ERR, __VA_ARGS__); \
   writeException(x, y, type, packet, size); \
 } while(0)
 
@@ -187,8 +187,8 @@ typedef struct Connection {
   BrailleWindow brailleWindow;
   BrlBufState brlbufstate;
   pthread_mutex_t brlMutex;
-  KeyrangeList *unmaskedKeys;
-  pthread_mutex_t maskMutex;
+  KeyrangeList *acceptedKeys;
+  pthread_mutex_t acceptedKeysMutex;
   time_t upTime;
   Packet packet;
 } Connection;
@@ -230,7 +230,7 @@ static pthread_mutex_t rawMutex;
 static Connection *rawConnection = NULL;
 static Connection *suspendConnection = NULL;
 
-/* mutex lock order is connectionsMutex first, then rawMutex, then (maskMutex
+/* mutex lock order is connectionsMutex first, then rawMutex, then (acceptedKeysMutex
  * or brlMutex) then driverMutex */
 
 static Tty notty;
@@ -296,12 +296,14 @@ static int refresh;
 static WSADATA wsadata;
 #endif /* __MINGW32__ */
 
+static unsigned char cursorShape;
+
 /****************************************************************************/
 /** SOME PROTOTYPES                                                        **/
 /****************************************************************************/
 
 extern void processParameters(char ***values, const char *const *names, const char *description, char *optionParameters, char *configuredParameters, const char *environmentVariable);
-static int initializeUnmaskedKeys(Connection *c);
+static int initializeAcceptedKeys(Connection *c);
 
 /****************************************************************************/
 /** DRIVER CAPABILITIES                                                    **/
@@ -527,7 +529,7 @@ int allocBrailleWindow(BrailleWindow *brailleWindow)
   if (!(brailleWindow->andAttr = malloc(displaySize))) goto outText;
   if (!(brailleWindow->orAttr = malloc(displaySize))) goto outAnd;
 
-  wmemset(brailleWindow->text, L' ', displaySize);
+  wmemset(brailleWindow->text, WC_C(' '), displaySize);
   memset(brailleWindow->andAttr, 0xFF, displaySize);
   memset(brailleWindow->orAttr, 0x00, displaySize);
   brailleWindow->cursor = 0;
@@ -567,45 +569,14 @@ void copyBrailleWindow(BrailleWindow *dest, const BrailleWindow *src)
 /* Returns the braille dots corresponding to a BrailleWindow structure */
 /* No allocation of buf is performed */
 void getDots(const BrailleWindow *brailleWindow, unsigned char *buf)
-{
-  int i, c;
-  wchar_t wc;
-  for (i=0; i<displaySize; i++) {
-    wc = brailleWindow->text[i];
-    if ((wc >= BRLAPI_UC_ROW) && (wc <= (BRLAPI_UC_ROW | 0XFF)))
-      buf[i] =
-	(wc&(1<<(1-1))?BRLAPI_DOT1:0) |
-	(wc&(1<<(2-1))?BRLAPI_DOT2:0) |
-	(wc&(1<<(3-1))?BRLAPI_DOT3:0) |
-	(wc&(1<<(4-1))?BRLAPI_DOT4:0) |
-	(wc&(1<<(5-1))?BRLAPI_DOT5:0) |
-	(wc&(1<<(6-1))?BRLAPI_DOT6:0) |
-	(wc&(1<<(7-1))?BRLAPI_DOT7:0) |
-	(wc&(1<<(8-1))?BRLAPI_DOT8:0);
-    else
-      if ((c = convertWcharToChar(wc)) != EOF)
-        buf[i] = textTable[c];
-      else
-        /* TODO: "invalid" pattern */
-        buf[i] = textTable[(unsigned char) '?'];
-    buf[i] = (buf[i] & brailleWindow->andAttr[i]) | brailleWindow->orAttr[i];
-  }
-  if (brailleWindow->cursor) buf[brailleWindow->cursor-1] |= cursorDots();
-}
-
-/* Function: getText */
-/* Returns the text corresponding to a BrailleWindow structure */
-/* No allocation of buf is performed */
-void getText(const BrailleWindow *brailleWindow, unsigned char *buf)
 {
   int i;
-  int c;
+  unsigned char c;
   for (i=0; i<displaySize; i++) {
-    if ((c = convertWcharToChar(brailleWindow->text[i])) == EOF)
-      buf[i] = '?';
-    else
-      buf[i] = c;
+    c = convertWcharToDots(textTable, brailleWindow->text[i]);
+    buf[i] = (c & brailleWindow->andAttr[i]) | brailleWindow->orAttr[i];
   }
+  if (brailleWindow->cursor) buf[brailleWindow->cursor-1] |= cursorShape;
 }
 
 static void handleResize(BrailleDisplay *brl)
@@ -634,9 +605,9 @@ static Connection *createConnection(FileDescriptor fd, time_t currentTime)
   pthread_mutexattr_init(&mattr);
   pthread_mutexattr_settype(&mattr, PTHREAD_MUTEX_RECURSIVE);
   pthread_mutex_init(&c->brlMutex,&mattr);
-  pthread_mutex_init(&c->maskMutex,&mattr);
+  pthread_mutex_init(&c->acceptedKeysMutex,&mattr);
   c->how = 0;
-  c->unmaskedKeys = NULL;
+  c->acceptedKeys = NULL;
   c->upTime = currentTime;
   c->brailleWindow.text = NULL;
   c->brailleWindow.andAttr = NULL;
@@ -657,11 +628,14 @@ out:
 /* Frees all resources associated to a connection */
 static void freeConnection(Connection *c)
 {
-  if (c->fd>=0) closeFileDescriptor(c->fd);
+  if (c->fd>=0) {
+    if (c->auth != 1) unauthConnections--;
+    closeFileDescriptor(c->fd);
+  }
   pthread_mutex_destroy(&c->brlMutex);
-  pthread_mutex_destroy(&c->maskMutex);
+  pthread_mutex_destroy(&c->acceptedKeysMutex);
   freeBrailleWindow(&c->brailleWindow);
-  freeKeyrangeList(&c->unmaskedKeys);
+  freeKeyrangeList(&c->acceptedKeys);
   free(c);
 }
 
@@ -811,10 +785,10 @@ static int handleEnterTtyMode(Connection *c, brlapi_packetType_t type, brlapi_pa
     how = BRL_KEYCODES;
   }
   freeBrailleWindow(&c->brailleWindow); /* In case of multiple enterTtyMode requests */
-  if ((initializeUnmaskedKeys(c)==-1) || (allocBrailleWindow(&c->brailleWindow)==-1)) {
+  if ((initializeAcceptedKeys(c)==-1) || (allocBrailleWindow(&c->brailleWindow)==-1)) {
     LogPrint(LOG_WARNING,"Failed to allocate some ressources");
-    freeKeyrangeList(&c->unmaskedKeys);
-    WERR(c->fd,BRLAPI_ERROR_NOMEM, "no memory for unmasked keys");
+    freeKeyrangeList(&c->acceptedKeys);
+    WERR(c->fd,BRLAPI_ERROR_NOMEM, "no memory for accepted keys");
     return 0;
   }
   pthread_mutex_lock(&connectionsMutex);
@@ -912,7 +886,7 @@ static void doLeaveTty(Connection *c)
   __removeConnection(c);
   __addConnection(c,notty.connections);
   pthread_mutex_unlock(&connectionsMutex);
-  freeKeyrangeList(&c->unmaskedKeys);
+  freeKeyrangeList(&c->acceptedKeys);
   freeBrailleWindow(&c->brailleWindow);
 }
 
@@ -936,20 +910,20 @@ static int handleKeyRanges(Connection *c, brlapi_packetType_t type, brlapi_packe
   CHECKERR(!c->raw,BRLAPI_ERROR_ILLEGAL_INSTRUCTION,"not allowed in raw mode");
   CHECKERR(c->tty,BRLAPI_ERROR_ILLEGAL_INSTRUCTION,"not allowed out of tty mode");
   CHECKERR(!(size%2*sizeof(brlapi_keyCode_t)),BRLAPI_ERROR_INVALID_PACKET,"wrong packet size");
-  pthread_mutex_lock(&c->maskMutex);
+  pthread_mutex_lock(&c->acceptedKeysMutex);
   for (i=0; i<size/(2*sizeof(brlapi_keyCode_t)); i++) {
     x = ((brlapi_keyCode_t)ntohl(ints[i][0]) << 32) | ntohl(ints[i][1]);
     y = ((brlapi_keyCode_t)ntohl(ints[i][2]) << 32) | ntohl(ints[i][3]);
     LogPrint(LOG_DEBUG,"range: [%016"BRLAPI_PRIxKEYCODE"..%016"BRLAPI_PRIxKEYCODE"]",x,y);
-    if (type==BRLAPI_PACKET_IGNOREKEYRANGES) res = removeKeyrange(x,y,&c->unmaskedKeys);
-    else res = addKeyrange(x,y,&c->unmaskedKeys);
+    if (type==BRLAPI_PACKET_IGNOREKEYRANGES) res = removeKeyrange(x,y,&c->acceptedKeys);
+    else res = addKeyrange(x,y,&c->acceptedKeys);
     if (res==-1) {
       /* XXX: humf, in the middle of keycode updates :( */
       WERR(c->fd,BRLAPI_ERROR_NOMEM,"no memory for key range");
       break;
     }
   }
-  pthread_mutex_unlock(&c->maskMutex);
+  pthread_mutex_unlock(&c->acceptedKeysMutex);
   if (!res) writeAck(c->fd);
   return 0;
 }
@@ -1215,6 +1189,7 @@ static int handleUnauthorizedConnection(Connection *c, brlapi_packetType_t type,
       /* TODO: move this inside auth.c */
       if (authDescriptor && authPerform(authDescriptor, c->fd)) {
 	authPacket->type[nbmethods++] = htonl(BRLAPI_AUTH_NONE);
+	unauthConnections--;
 	c->auth = 1;
       } else {
 	if (isAbsolutePath(auth))
@@ -1335,7 +1310,6 @@ static int processRequest(Connection *c, PacketHandlers *handlers)
       LogPrint(LOG_DEBUG,"Client on fd %"PRIFD" did not give up control of tty %#010x properly",c->fd,c->tty->number);
       doLeaveTty(c);
     }
-    if (c->auth!=1) unauthConnections--;
     return 1;
   }
   size = c->packet.header.size;
@@ -2185,25 +2159,25 @@ static void *server(void *arg)
 /** MISCELLANEOUS FUNCTIONS                                                **/
 /****************************************************************************/
 
-/* Function : initializeUnmaskedKeys */
+/* Function : initializeAcceptedKeys */
 /* Specify which keys should be passed to the client by default, as soon */
 /* as it controls the tty */
 /* If client asked for commands, one lets it process routing cursor */
 /* and screen-related commands */
 /* If the client is interested in braille codes, one passes it nothing */
 /* to let the user read the screen in case theree is an error */
-static int initializeUnmaskedKeys(Connection *c)
+static int initializeAcceptedKeys(Connection *c)
 {
   if (c==NULL) return 0;
   if (c->how==BRL_KEYCODES) return 0;
-  if (addKeyrange(0,BRLAPI_KEY_MAX,&c->unmaskedKeys)==-1) return -1;
-  if (removeKeyrange(BRLAPI_KEY_TYPE_CMD|BRLAPI_KEY_CMD_NOOP,BRLAPI_KEY_TYPE_CMD|BRLAPI_KEY_CMD_NOOP|BRLAPI_KEY_FLAGS_MASK,&c->unmaskedKeys)==-1) return -1;
-  if (removeKeyrange(BRLAPI_KEY_TYPE_CMD|BRLAPI_KEY_CMD_SWITCHVT_PREV,BRLAPI_KEY_TYPE_CMD|BRLAPI_KEY_CMD_SWITCHVT_NEXT|BRLAPI_KEY_FLAGS_MASK,&c->unmaskedKeys)==-1) return -1;
-  if (removeKeyrange(BRLAPI_KEY_TYPE_CMD|BRLAPI_KEY_CMD_RESTARTBRL,BRLAPI_KEY_TYPE_CMD|BRLAPI_KEY_CMD_RESTARTSPEECH|BRLAPI_KEY_FLAGS_MASK,&c->unmaskedKeys)==-1) return -1;
-  if (removeKeyrange(BRLAPI_KEY_TYPE_CMD|BRLAPI_KEY_CMD_SWITCHVT,BRLAPI_KEY_TYPE_CMD|BRLAPI_KEY_CMD_SWITCHVT|BRLAPI_KEY_CMD_ARG_MASK|BRLAPI_KEY_FLAGS_MASK,&c->unmaskedKeys)==-1) return -1;
-  if (removeKeyrange(BRLAPI_KEY_TYPE_CMD|BRLAPI_KEY_CMD_PASSXT,BRLAPI_KEY_TYPE_CMD|BRLAPI_KEY_CMD_PASSXT|BRLAPI_KEY_CMD_ARG_MASK|BRLAPI_KEY_FLAGS_MASK,&c->unmaskedKeys)==-1) return -1;
-  if (removeKeyrange(BRLAPI_KEY_TYPE_CMD|BRLAPI_KEY_CMD_PASSAT,BRLAPI_KEY_TYPE_CMD|BRLAPI_KEY_CMD_PASSAT|BRLAPI_KEY_CMD_ARG_MASK|BRLAPI_KEY_FLAGS_MASK,&c->unmaskedKeys)==-1) return -1;
-  if (removeKeyrange(BRLAPI_KEY_TYPE_CMD|BRLAPI_KEY_CMD_PASSPS2,BRLAPI_KEY_TYPE_CMD|BRLAPI_KEY_CMD_PASSPS2|BRLAPI_KEY_CMD_ARG_MASK|BRLAPI_KEY_FLAGS_MASK,&c->unmaskedKeys)==-1) return -1;
+  if (addKeyrange(0,BRLAPI_KEY_MAX,&c->acceptedKeys)==-1) return -1;
+  if (removeKeyrange(BRLAPI_KEY_TYPE_CMD|BRLAPI_KEY_CMD_NOOP,BRLAPI_KEY_TYPE_CMD|BRLAPI_KEY_CMD_NOOP|BRLAPI_KEY_FLAGS_MASK,&c->acceptedKeys)==-1) return -1;
+  if (removeKeyrange(BRLAPI_KEY_TYPE_CMD|BRLAPI_KEY_CMD_SWITCHVT_PREV,BRLAPI_KEY_TYPE_CMD|BRLAPI_KEY_CMD_SWITCHVT_NEXT|BRLAPI_KEY_FLAGS_MASK,&c->acceptedKeys)==-1) return -1;
+  if (removeKeyrange(BRLAPI_KEY_TYPE_CMD|BRLAPI_KEY_CMD_RESTARTBRL,BRLAPI_KEY_TYPE_CMD|BRLAPI_KEY_CMD_RESTARTSPEECH|BRLAPI_KEY_FLAGS_MASK,&c->acceptedKeys)==-1) return -1;
+  if (removeKeyrange(BRLAPI_KEY_TYPE_CMD|BRLAPI_KEY_CMD_SWITCHVT,BRLAPI_KEY_TYPE_CMD|BRLAPI_KEY_CMD_SWITCHVT|BRLAPI_KEY_CMD_ARG_MASK|BRLAPI_KEY_FLAGS_MASK,&c->acceptedKeys)==-1) return -1;
+  if (removeKeyrange(BRLAPI_KEY_TYPE_CMD|BRLAPI_KEY_CMD_PASSXT,BRLAPI_KEY_TYPE_CMD|BRLAPI_KEY_CMD_PASSXT|BRLAPI_KEY_CMD_ARG_MASK|BRLAPI_KEY_FLAGS_MASK,&c->acceptedKeys)==-1) return -1;
+  if (removeKeyrange(BRLAPI_KEY_TYPE_CMD|BRLAPI_KEY_CMD_PASSAT,BRLAPI_KEY_TYPE_CMD|BRLAPI_KEY_CMD_PASSAT|BRLAPI_KEY_CMD_ARG_MASK|BRLAPI_KEY_FLAGS_MASK,&c->acceptedKeys)==-1) return -1;
+  if (removeKeyrange(BRLAPI_KEY_TYPE_CMD|BRLAPI_KEY_CMD_PASSPS2,BRLAPI_KEY_TYPE_CMD|BRLAPI_KEY_CMD_PASSPS2|BRLAPI_KEY_CMD_ARG_MASK|BRLAPI_KEY_FLAGS_MASK,&c->acceptedKeys)==-1) return -1;
   return 0;
 }
 
@@ -2255,36 +2229,21 @@ static inline void setCurrentRootTty(void) {
 }
 
 /* Function : api_writeWindow */
-static void api_writeWindow(BrailleDisplay *brl)
+static int api_writeWindow(BrailleDisplay *brl, const wchar_t *text)
 {
+  int ok = 1;
   setCurrentRootTty();
   pthread_mutex_lock(&connectionsMutex);
   pthread_mutex_lock(&rawMutex);
   if (!offline && !suspendConnection && !rawConnection && !whoFillsTty(&ttys)) {
     last_conn_write=NULL;
     pthread_mutex_lock(&driverMutex);
-    trueBraille->writeWindow(brl);
-    pthread_mutex_unlock(&driverMutex);
-  }
-  pthread_mutex_unlock(&rawMutex);
-  pthread_mutex_unlock(&connectionsMutex);
-}
-
-/* Function : api_writeVisual */
-static void api_writeVisual(BrailleDisplay *brl)
-{
-  setCurrentRootTty();
-  pthread_mutex_lock(&connectionsMutex);
-  pthread_mutex_lock(&rawMutex);
-  if (trueBraille->writeVisual &&
-      !offline && !suspendConnection && !rawConnection && !whoFillsTty(&ttys)) {
-    last_conn_write=NULL;
-    pthread_mutex_lock(&driverMutex);
-    trueBraille->writeVisual(brl);
+    if (!trueBraille->writeWindow(brl, text)) ok = 0;
     pthread_mutex_unlock(&driverMutex);
   }
   pthread_mutex_unlock(&rawMutex);
   pthread_mutex_unlock(&connectionsMutex);
+  return ok;
 }
 
 /* Function: whoGetsKey */
@@ -2295,9 +2254,9 @@ static Connection *whoGetsKey(Tty *tty, brlapi_keyCode_t code, unsigned int how)
   Tty *t;
   int passKey;
   for (c=tty->connections->next; c!=tty->connections; c = c->next) {
-    pthread_mutex_lock(&c->maskMutex);
-    passKey = (c->how==how) && (inKeyrangeList(c->unmaskedKeys,code) != NULL);
-    pthread_mutex_unlock(&c->maskMutex);
+    pthread_mutex_lock(&c->acceptedKeysMutex);
+    passKey = (c->how==how) && (inKeyrangeList(c->acceptedKeys,code) != NULL);
+    pthread_mutex_unlock(&c->acceptedKeysMutex);
     if (passKey) goto found;
   }
   c = NULL;
@@ -2319,6 +2278,8 @@ static int api_readCommand(BrailleDisplay *brl, BRL_DriverCommandContext caller)
   brlapi_packet_t packet;
   int keycode, command = EOF;
   brlapi_keyCode_t clientCode;
+  int ok = 1;
+  unsigned char newCursorShape;
 
   pthread_mutex_lock(&connectionsMutex);
   pthread_mutex_lock(&rawMutex);
@@ -2356,23 +2317,24 @@ static int api_readCommand(BrailleDisplay *brl, BRL_DriverCommandContext caller)
       refresh=1;
     }
     pthread_mutex_unlock(&driverMutex);
+    newCursorShape = cursorDots();
+    if (newCursorShape!=cursorShape) {
+      cursorShape = newCursorShape;
+      refresh = 1;
+    }
     if ((c->brlbufstate==TODISPLAY) || (refresh)) {
       unsigned char *oldbuf = disp->buffer, buf[displaySize];
       disp->buffer = buf;
-      if (trueBraille->writeVisual) {
-        getText(&c->brailleWindow, buf);
-        brl->cursor = c->brailleWindow.cursor-1;
-	pthread_mutex_lock(&driverMutex);
-        trueBraille->writeVisual(brl);
-	pthread_mutex_unlock(&driverMutex);
-      }
       getDots(&c->brailleWindow, buf);
       pthread_mutex_lock(&driverMutex);
-      trueBraille->writeWindow(brl);
+      brl->cursor = c->brailleWindow.cursor-1;
+      ok = trueBraille->writeWindow(brl, c->brailleWindow.text);
       pthread_mutex_unlock(&driverMutex);
       disp->buffer = oldbuf;
-      c->brlbufstate = FULL;
-      refresh=0;
+      if (ok) {
+	c->brlbufstate = FULL;
+	refresh=0;
+      }
     }
     pthread_mutex_unlock(&c->brlMutex);
   } else {
@@ -2386,6 +2348,11 @@ static int api_readCommand(BrailleDisplay *brl, BRL_DriverCommandContext caller)
     }
     pthread_mutex_unlock(&driverMutex);
   }
+  if (!ok) {
+    pthread_mutex_unlock(&rawMutex);
+    command = BRL_CMD_RESTARTBRL;
+    goto out;
+  }
   if (trueBraille->readKey) {
     pthread_mutex_lock(&driverMutex);
     res = trueBraille->readKey(brl);
@@ -2411,7 +2378,7 @@ static int api_readCommand(BrailleDisplay *brl, BRL_DriverCommandContext caller)
   clientCode = keycode;
   if (trueBraille->readKey && keycode != EOF && (c = whoGetsKey(&ttys,clientCode,BRL_KEYCODES))) {
     /* somebody gets the raw code */
-    LogPrint(LOG_DEBUG,"Transmitting unmasked key %lu",(unsigned long)keycode);
+    LogPrint(LOG_DEBUG,"Transmitting accepted key %lu",(unsigned long)keycode);
     writeKey(c->fd,clientCode);
     command = EOF;
   } else {
@@ -2426,7 +2393,7 @@ static int api_readCommand(BrailleDisplay *brl, BRL_DriverCommandContext caller)
       clientCode = cmdBrlttyToBrlapi(command);
       /* nobody needs the raw code */
       if ((c = whoGetsKey(&ttys,clientCode,BRL_COMMANDS))) {
-        LogPrint(LOG_DEBUG,"Transmitting unmasked command %lx as client code %016"BRLAPI_PRIxKEYCODE,(unsigned long)command, clientCode);
+        LogPrint(LOG_DEBUG,"Transmitting accepted command %lx as client code %016"BRLAPI_PRIxKEYCODE,(unsigned long)command, clientCode);
 	writeKey(c->fd,clientCode);
         command = EOF;
       }
@@ -2492,7 +2459,6 @@ void api_link(BrailleDisplay *brl)
   refresh=1;
   memcpy(&ApiBraille,braille,sizeof(BrailleDriver));
   ApiBraille.writeWindow=api_writeWindow;
-  ApiBraille.writeVisual=api_writeVisual;
   ApiBraille.readCommand=api_readCommand;
   ApiBraille.readKey = NULL;
   ApiBraille.keyToCommand = NULL;
