@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -21,13 +21,12 @@
 
 #include "prologue.h"
 
-#include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 #include <string.h>
 #include <strings.h>
 #include <errno.h>
 
+#include "program.h"
 #include "options.h"
 #include "brl.h"
 #include "tbl.h"
@@ -41,39 +40,64 @@ static BrailleDisplay brl;
 
 static char *opt_brailleDevice;
 static char *opt_libraryDirectory;
+static char *opt_writableDirectory;
 static char *opt_dataDirectory;
 
-BEGIN_OPTION_TABLE
-  {"device", "device", 'd', 0, 0,
-   &opt_brailleDevice, BRAILLE_DEVICE,
-   "Path to device for accessing braille display."},
+BEGIN_OPTION_TABLE(programOptions)
+  { .letter = 'D',
+    .word = "data-directory",
+    .flags = OPT_Hidden,
+    .argument = "directory",
+    .setting.string = &opt_dataDirectory,
+    .defaultSetting = DATA_DIRECTORY,
+    .description = "Path to directory for driver help and configuration files."
+  },
 
-  {"data-directory", "directory", 'D', 0, 0,
-   &opt_dataDirectory, DATA_DIRECTORY,
-   "Path to directory for driver help and configuration files."},
+  { .letter = 'L',
+    .word = "library-directory",
+    .flags = OPT_Hidden,
+    .argument = "directory",
+    .setting.string = &opt_libraryDirectory,
+    .defaultSetting = LIBRARY_DIRECTORY,
+    .description = "Path to directory for loading drivers."
+  },
 
-  {"library-directory", "directory", 'L', 0, 0,
-   &opt_libraryDirectory, LIBRARY_DIRECTORY,
-   "Path to directory for loading drivers."},
+  { .letter = 'W',
+    .word = "writable-directory",
+    .flags = OPT_Hidden,
+    .argument = strtext("directory"),
+    .setting.string = &opt_writableDirectory,
+    .defaultSetting = WRITABLE_DIRECTORY,
+    .description = strtext("Path to directory which can be written to.")
+  },
+
+  { .letter = 'd',
+    .word = "device",
+    .argument = "device",
+    .setting.string = &opt_brailleDevice,
+    .defaultSetting = BRAILLE_DEVICE,
+    .description = "Path to device for accessing braille display."
+  },
 END_OPTION_TABLE
 
-void
+int
 message (const char *string, short flags) {
-  int length = strlen(string);
-  int limit = brl.x * brl.y;
+  size_t length = strlen(string);
+  size_t size = brl.x * brl.y;
+  char buffer[size];
 
   clearStatusCells(&brl);
 
-  memset(brl.buffer, ' ', brl.x*brl.y);
+  memset(buffer, ' ', size);
   while (length) {
-    int count = (length <= limit)? length: (limit - 1);
+    int count = (length <= size)? length: (size - 1);
     int index;
-    for (index=0; index<count; brl.buffer[index++]=*string++);
-    if (length -= count)
-      brl.buffer[(brl.x * brl.y) - 1] = '-';
-    else
-      while (index < limit) brl.buffer[index++] = ' ';
-    writeBrailleBuffer(&brl);
+
+    for (index=0; index<count; buffer[index++]=*string++);
+    if (length -= count) {
+      buffer[(count = size) - 1] = WC_C('-');
+    }
+    if (!writeBrailleText(&brl, buffer, count)) return 0;
 
     if (length) {
       int timer = 0;
@@ -84,6 +108,8 @@ message (const char *string, short flags) {
       }
     }
   }
+
+  return 1;
 }
 
 int
@@ -92,10 +118,26 @@ main (int argc, char *argv[]) {
   const char *driver = NULL;
   void *object;
 
-  processOptions(optionTable, optionCount,
-                 "brltest", &argc, &argv,
-                 NULL, NULL, NULL,
-                 "[driver [parameter=value ...]]");
+  {
+    static const OptionsDescriptor descriptor = {
+      OPTION_TABLE(programOptions),
+      .applicationName = "brltest",
+      .argumentsSummary = "[driver [parameter=value ...]]"
+    };
+    processOptions(&descriptor, &argc, &argv);
+  }
+
+  {
+    char **const paths[] = {
+      &opt_libraryDirectory,
+      &opt_writableDirectory,
+      &opt_dataDirectory,
+      NULL
+    };
+    fixInstallPaths(paths);
+  }
+
+  writableDirectory = opt_writableDirectory;
 
   if (argc) {
     driver = *argv++;
@@ -153,15 +195,15 @@ main (int argc, char *argv[]) {
       makeUntextTable();
       initializeBrailleDisplay(&brl);
       brl.dataDirectory = opt_dataDirectory;
-      braille->identify();		/* start-up messages */
-      if (braille->open(&brl, parameterSettings, opt_brailleDevice)) {
-        if (allocateBrailleBuffer(&brl)) {
+      identifyBrailleDriver(braille, 0);		/* start-up messages */
+      if (braille->construct(&brl, parameterSettings, opt_brailleDevice)) {
+        if (ensureBrailleBuffer(&brl, LOG_INFO)) {
 #ifdef ENABLE_LEARN_MODE
           learnMode(&brl, updateInterval, 10000);
 #else /* ENABLE_LEARN_MODE */
           message("braille test", 0);
 #endif /* ENABLE_LEARN_MODE */
-          braille->close(&brl);		/* finish with the display */
+          braille->destruct(&brl);		/* finish with the display */
           status = 0;
         } else {
           LogPrint(LOG_ERR, "can't allocate braille buffer.");
@@ -191,7 +233,3 @@ int
 currentVirtualTerminal (void) {
   return 0;
 }
-int
-insertString (const char *string) {
-  return 0;
-}
