@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -20,6 +20,8 @@
 
 #include <sys/time.h>
 
+#include "brlapi_keycodes.h"
+
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
@@ -41,7 +43,10 @@ typedef struct {
   struct timeval time;
 } RepeatState;
 extern void resetRepeatState (RepeatState *state);
-extern void handleRepeatFlags (int *command, RepeatState *state, int delay, int interval);
+extern void handleRepeatFlags (int *command, RepeatState *state, int panning, int delay, int interval);
+
+extern brlapi_keyCode_t cmdBrlttyToBrlapi (int command);
+extern int cmdBrlapiToBrltty (brlapi_keyCode_t code);
 
 #ifdef __cplusplus
 }
