@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -22,11 +22,16 @@
 extern "C" {
 #endif /* __cplusplus */
 
-extern void *compileContractionTable (const char *fileName);
-extern int destroyContractionTable (void *contractionTable);
+typedef struct ContractionTableStruct ContractionTable;
+
+#define CTB_NO_OFFSET -1
+#define CTB_NO_CURSOR -1
+
+extern ContractionTable *compileContractionTable (const char *fileName);
+extern int destroyContractionTable (ContractionTable *contractionTable);
 extern int contractText (
-  void *contractionTable, /* Pointer to translation table */
-  const unsigned char *inputBuffer, /* What is to be translated */
+  ContractionTable *contractionTable, /* Pointer to translation table */
+  const wchar_t *inputBuffer, /* What is to be translated */
   int *inputLength, /* Its length */
   unsigned char *outputBuffer, /* Where the translation is to go */
   int *outputLength, /* length of this area */
