@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -23,60 +23,55 @@
 
 #include "scr.h"
 #include "tunes.h"
+#include "charset.h"
 #include "cut.h"
 
 /* Global state variables */
-unsigned char *cutBuffer = NULL;
+wchar_t *cutBuffer = NULL;
 size_t cutLength = 0;
 
 static int beginColumn = 0;
 static int beginRow = 0;
 static int beginOffset = -1;
 
-static unsigned char *
+static wchar_t *
+mallocWchars (size_t count) {
+  return malloc(count * sizeof(wchar_t));
+}
+
+static wchar_t *
 cut (size_t *length, int fromColumn, int fromRow, int toColumn, int toRow) {
-  unsigned char *newBuffer = NULL;
+  wchar_t *newBuffer = NULL;
   int columns = toColumn - fromColumn + 1;
   int rows = toRow - fromRow + 1;
 
   if ((columns >= 1) && (rows >= 1) && (beginOffset >= 0)) {
-    unsigned char *fromBuffer = malloc(rows * columns);
+    wchar_t *fromBuffer = mallocWchars(rows * columns);
     if (fromBuffer) {
-      unsigned char *toBuffer = malloc(rows * (columns + 1));
+      wchar_t *toBuffer = mallocWchars(rows * (columns + 1));
       if (toBuffer) {
-        if (readScreen(fromColumn, fromRow, columns, rows, fromBuffer, SCR_TEXT)) {
-          unsigned char *fromAddress = fromBuffer;
-          unsigned char *toAddress = toBuffer;
+        if (readScreenText(fromColumn, fromRow, columns, rows, fromBuffer)) {
+          wchar_t *fromAddress = fromBuffer;
+          wchar_t *toAddress = toBuffer;
           int row;
 
-          /* remove spaces at end of line, add return (except to last line),
-           * and possibly remove non-printables... if any
-           */
-          for (row=fromRow; row<=toRow; row++) {
-            int spaces = 0;
+          for (row=fromRow; row<=toRow; row+=1) {
             int column;
 
-            for (column=fromColumn; column<=toColumn; column++, fromAddress++) {
-              if (iscntrl(*fromAddress) || isspace(*fromAddress)) {
-                spaces++;
-              } else {
-                while (spaces) {
-                  *(toAddress++) = ' ';
-                  spaces--;
-                }
-
-                *(toAddress++) = *fromAddress;
-              }
+            for (column=fromColumn; column<=toColumn; column+=1) {
+              wchar_t character = *fromAddress++;
+              if (iswcntrl(character) || iswspace(character)) character = WC_C(' ');
+              *toAddress++ = character;
             }
 
-            if (row != toRow) *(toAddress++) = '\r';
+            if (row != toRow) *toAddress++ = WC_C('\r');
           }
 
           /* make a new permanent buffer of just the right size */
           {
             size_t newLength = toAddress - toBuffer;
-            if ((newBuffer = malloc(newLength))) {
-              memcpy(newBuffer, toBuffer, (*length = newLength));
+            if ((newBuffer = mallocWchars(newLength))) {
+              wmemcpy(newBuffer, toBuffer, (*length = newLength));
             }
           }
         }
@@ -92,14 +87,14 @@ cut (size_t *length, int fromColumn, int fromRow, int toColumn, int toRow) {
 }
 
 static int
-append (unsigned char *buffer, size_t length) {
+append (wchar_t *buffer, size_t length) {
   if (cutBuffer) {
     size_t newLength = beginOffset + length;
-    unsigned char *newBuffer = malloc(newLength);
+    wchar_t *newBuffer = mallocWchars(newLength);
     if (!newBuffer) return 0;
 
-    memcpy(newBuffer, cutBuffer, beginOffset);
-    memcpy(newBuffer+beginOffset, buffer, length);
+    wmemcpy(newBuffer, cutBuffer, beginOffset);
+    wmemcpy(newBuffer+beginOffset, buffer, length);
 
     free(buffer);
     free(cutBuffer);
@@ -116,13 +111,17 @@ append (unsigned char *buffer, size_t length) {
 }
 
 void
-cutBegin (int column, int row) {
+cutClear (void) {
   if (cutBuffer) {
     free(cutBuffer);
     cutBuffer = NULL;
   }
   cutLength = 0;
+}
 
+void
+cutBegin (int column, int row) {
+  cutClear();
   cutAppend(column, row);
 }
 
@@ -137,9 +136,41 @@ cutAppend (int column, int row) {
 int
 cutRectangle (int column, int row) {
   size_t length;
-  unsigned char *buffer = cut(&length, beginColumn, beginRow, column, row);
+  wchar_t *buffer = cut(&length, beginColumn, beginRow, column, row);
 
   if (buffer) {
+    {
+      const wchar_t *from = buffer;
+      const wchar_t *end = from + length;
+      wchar_t *to = buffer;
+      int spaces = 0;
+
+      while (from != end) {
+        wchar_t character = *from++;
+
+        switch (character) {
+          case WC_C(' '):
+            spaces += 1;
+            continue;
+
+          case WC_C('\r'):
+            spaces = 0;
+
+          default:
+            break;
+        }
+
+        while (spaces) {
+          *to++ = WC_C(' ');
+          spaces -= 1;
+        }
+
+        *to++ = character;
+      }
+
+      length = to - buffer;
+    }
+
     if (append(buffer, length)) return 1;
     free(buffer);
   }
@@ -154,14 +185,14 @@ cutLine (int column, int row) {
   {
     int rightColumn = screen.cols - 1;
     size_t length;
-    unsigned char *buffer = cut(&length, 0, beginRow, rightColumn, row);
+    wchar_t *buffer = cut(&length, 0, beginRow, rightColumn, row);
 
     if (buffer) {
       if (column < rightColumn) {
-        unsigned char *start = buffer + length;
+        wchar_t *start = buffer + length;
         while (start != buffer) {
-          if (*--start == '\r') {
-            ++start;
+          if (*--start == WC_C('\r')) {
+            start += 1;
             break;
           }
         }
@@ -173,10 +204,49 @@ cutLine (int column, int row) {
       }
 
       if (beginColumn) {
-        unsigned char *start = memchr(buffer, '\r', length);
+        wchar_t *start = wmemchr(buffer, WC_C('\r'), length);
         if (!start) start = buffer + length;
         if ((start - buffer) > beginColumn) start = buffer + beginColumn;
-        if (start != buffer) memmove(buffer, start, (length -= start - buffer));
+        if (start != buffer) wmemmove(buffer, start, (length -= start - buffer));
+      }
+
+      {
+        const wchar_t *from = buffer;
+        const wchar_t *end = from + length;
+        wchar_t *to = buffer;
+        int spaces = 0;
+        int newlines = 0;
+
+        while (from != end) {
+          wchar_t character = *from++;
+
+          switch (character) {
+            case WC_C(' '):
+              spaces += 1;
+              continue;
+
+            case WC_C('\r'):
+              newlines += 1;
+              continue;
+
+            default:
+              break;
+          }
+
+          if (newlines) {
+            if ((newlines > 1) || (spaces > 0)) spaces = 1;
+            newlines = 0;
+          }
+
+          while (spaces) {
+            *to++ = WC_C(' ');
+            spaces -= 1;
+          }
+
+          *to++ = character;
+        }
+
+        length = to - buffer;
       }
 
       if (append(buffer, length)) return 1;
@@ -189,5 +259,14 @@ cutLine (int column, int row) {
 
 int
 cutPaste (void) {
-  return cutBuffer && insertCharacters((char *)cutBuffer, cutLength);
+  if (!cutLength) return 0;
+
+  {
+    int i;
+    for (i=0; i<cutLength; ++i)
+      if (!insertScreenKey(cutBuffer[i]))
+        return 0;
+  }
+
+  return 1;
 }
