@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -22,7 +22,7 @@
 extern "C" {
 #endif /* __cplusplus */
 
-extern unsigned char *cutBuffer;
+extern wchar_t *cutBuffer;
 extern size_t cutLength;
 
 extern void cutBegin (int column, int row);
