@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -25,7 +25,7 @@ extern "C" {
 /* message.h - send a message to Braille and speech */
 
 /* Prototype: */
-extern void message (const char *, short);
+extern int message (const char *string, short flags);
 
 
 /* Flags for the second argument: */
