@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,12 +15,8 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif /* HAVE_CONFIG_H */
+#include "prologue.h"
 
-#include <stdlib.h>
-#include <unistd.h>
 #include <string.h>
 #include <errno.h>
 
@@ -29,10 +25,12 @@
 #include "system.h"
 #include "notes.h"
 
-static MidiDevice *midi = NULL;
-static int channelNumber = 0;
+struct NoteDeviceStruct {
+  MidiDevice *midi;
+  int channelNumber;
+};
 
-const char *midiInstrumentTable[] = {
+const char *const midiInstrumentTable[] = {
 /* Piano */
   /* 00 */ "Acoustic Grand Piano",
   /* 01 */ "Bright Acoustic Piano",
@@ -178,53 +176,63 @@ const char *midiInstrumentTable[] = {
   /* 7E */ "Applause",
   /* 7F */ "Gunshot"
 };
-const unsigned int midiInstrumentCount = sizeof(midiInstrumentTable) / sizeof(midiInstrumentTable[0]);
+const unsigned int midiInstrumentCount = ARRAY_COUNT(midiInstrumentTable);
 
-static int openMidi (int errorLevel) {
-   if (!midi) {
-      if (!(midi = openMidiDevice(errorLevel, opt_midiDevice))) {
-         LogPrint(LOG_DEBUG, "Cannot open MIDI.");
-         return 0;
-      }
-      LogPrint(LOG_DEBUG, "MIDI opened.");
-   }
-   setMidiInstrument(midi, channelNumber, prefs.midiInstrument);
-   return 1;
+static NoteDevice *
+midiConstruct (int errorLevel) {
+  NoteDevice *device;
+
+  if ((device = malloc(sizeof(*device)))) {
+    if ((device->midi = openMidiDevice(errorLevel, opt_midiDevice))) {
+      device->channelNumber = 0;
+      setMidiInstrument(device->midi, device->channelNumber, prefs.midiInstrument);
+
+      LogPrint(LOG_DEBUG, "MIDI enabled");
+      return device;
+    }
+
+    free(device);
+  } else {
+    LogError("malloc");
+  }
+
+  LogPrint(LOG_DEBUG, "MIDI not available");
+  return NULL;
 }
 
-static int playMidi (int note, int duration) {
-   if (midi) {
-      beginMidiBlock(midi);
-      if (note) {
-	 LogPrint(LOG_DEBUG, "Tone: msec=%d note=%d", duration, note);
-         startMidiNote(midi, channelNumber, note, prefs.midiVolume);
-	 insertMidiWait(midi, duration);
-         stopMidiNote(midi, channelNumber);
-      } else {
-	 LogPrint(LOG_DEBUG, "Tone: msec=%d", duration);
-	 insertMidiWait(midi, duration);
-      }
-      endMidiBlock(midi);
-      return 1;
-   }
-   return 0;
+static int
+midiPlay (NoteDevice *device, int note, int duration) {
+  beginMidiBlock(device->midi);
+
+  if (note) {
+    LogPrint(LOG_DEBUG, "tone: msec=%d note=%d", duration, note);
+    startMidiNote(device->midi, device->channelNumber, note, prefs.midiVolume);
+    insertMidiWait(device->midi, duration);
+    stopMidiNote(device->midi, device->channelNumber);
+  } else {
+    LogPrint(LOG_DEBUG, "tone: msec=%d", duration);
+    insertMidiWait(device->midi, duration);
+  }
+
+  endMidiBlock(device->midi);
+  return 1;
 }
 
-static int flushMidi (void) {
-   return flushMidiDevice(midi);
+static int
+midiFlush (NoteDevice *device) {
+  return flushMidiDevice(device->midi);
 }
 
-static void closeMidi (void) {
-   if (midi) {
-      closeMidiDevice(midi);
-      LogPrint(LOG_DEBUG, "MIDI closed.");
-      midi = NULL;
-   }
+static void
+midiDestruct (NoteDevice *device) {
+  closeMidiDevice(device->midi);
+  free(device);
+  LogPrint(LOG_DEBUG, "MIDI disabled");
 }
 
-const NoteGenerator midiNoteGenerator = {
-   openMidi,
-   playMidi,
-   flushMidi,
-   closeMidi
+const NoteMethods midiMethods = {
+  midiConstruct,
+  midiPlay,
+  midiFlush,
+  midiDestruct
 };
