@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -87,6 +87,32 @@ deallocateStrings (char **array) {
   free(array);
 }
 
+char *
+joinStrings (const char *const *strings, int count) {
+  char *string;
+  size_t length = 0;
+  size_t lengths[count];
+  int index;
+
+  for (index=0; index<count; index+=1) {
+    length += lengths[index] = strlen(strings[index]);
+  }
+
+  if ((string = malloc(length+1))) {
+    char *target = string;
+
+    for (index=0; index<count; index+=1) {
+      length = lengths[index];
+      memcpy(target, strings[index], length);
+      target += length;
+    }
+
+    *target = 0;
+  }
+
+  return string;
+}
+
 #if defined(HAVE_SYSLOG_H)
 static int syslogOpened = 0;
 
@@ -398,14 +424,9 @@ char *
 makePath (const char *directory, const char *file) {
   const int count = 3;
   const char *components[count];
-  int lengths[count];
-  const int last = count - 1;
-  int first = last;
-  int length = 0;
-  int index;
-  char *path;
+  int first = count;
 
-  components[last] = file;
+  components[--first] = file;
   if (!isAbsolutePath(file)) {
     if (directory && *directory) {
       if (!isPathDelimiter(directory[strlen(directory)-1])) components[--first] = "/";
@@ -413,20 +434,7 @@ makePath (const char *directory, const char *file) {
     }
   }
 
-  for (index=first; index<=last; ++index) {
-    length += lengths[index] = strlen(components[index]);
-  }
-
-  if ((path = malloc(length+1))) {
-    char *target = path;
-    for (index=first; index<=last; ++index) {
-      length = lengths[index];
-      memcpy(target, components[index], length);
-      target += length;
-    }
-    *target = 0;
-  }
-  return path;
+  return joinStrings(&components[first], count-first);
 }
 
 void
@@ -542,8 +550,8 @@ getUserDirectory (void) {
   char *home = getHomeDirectory();
   if (home) {
     char *directory = makePath(home, "." PACKAGE_NAME);
-    if (directory) return directory;
     free(home);
+    if (directory) return directory;
   }
   return NULL;
 }
@@ -990,3 +998,106 @@ int
 validateYesNo (unsigned int *value, const char *string) {
   return validateFlag(value, string, "yes", "no");
 }
+
+static void
+parseParameters (
+  char **values,
+  const char *const *names,
+  const char *qualifier,
+  const char *parameters
+) {
+  if (parameters && *parameters) {
+    char *copy = strdupWrapper(parameters);
+    char *name = copy;
+
+    while (1) {
+      char *end = strchr(name, ',');
+      int done = end == NULL;
+      if (!done) *end = 0;
+
+      if (*name) {
+        char *value = strchr(name, '=');
+        if (!value) {
+          LogPrint(LOG_ERR, "%s: %s", gettext("missing parameter value"), name);
+        } else if (value == name) {
+        noName:
+          LogPrint(LOG_ERR, "%s: %s", gettext("missing parameter name"), name);
+        } else {
+          int nameLength = value++ - name;
+          int eligible = 1;
+
+          if (qualifier) {
+            char *colon = memchr(name, ':', nameLength);
+            if (colon) {
+              int qualifierLength = colon - name;
+              int nameAdjustment = qualifierLength + 1;
+              eligible = 0;
+              if (!qualifierLength) {
+                LogPrint(LOG_ERR, "%s: %s", gettext("missing parameter qualifier"), name);
+              } else if (!(nameLength -= nameAdjustment)) {
+                goto noName;
+              } else if ((qualifierLength == strlen(qualifier)) &&
+                         (memcmp(name, qualifier, qualifierLength) == 0)) {
+                name += nameAdjustment;
+                eligible = 1;
+              }
+            }
+          }
+
+          if (eligible) {
+            unsigned int index = 0;
+            while (names[index]) {
+              if (strncasecmp(name, names[index], nameLength) == 0) {
+                free(values[index]);
+                values[index] = strdupWrapper(value);
+                break;
+              }
+              ++index;
+            }
+
+            if (!names[index]) {
+              LogPrint(LOG_ERR, "%s: %s", gettext("unsupported parameter"), name);
+            }
+          }
+        }
+      }
+
+      if (done) break;
+      name = end + 1;
+    }
+
+    free(copy);
+  }
+}
+
+char **
+getParameters (const char *const *names, const char *qualifier, const char *parameters) {
+  char **values;
+
+  if (!names) {
+    static const char *const noNames[] = {NULL};
+    names = noNames;
+  }
+
+  {
+    unsigned int count = 0;
+    while (names[count]) ++count;
+    values = mallocWrapper((count + 1) * sizeof(*values));
+    values[count] = NULL;
+    while (count--) values[count] = strdupWrapper("");
+  }
+
+  parseParameters(values, names, qualifier, parameters);
+  return values;
+}
+
+void
+logParameters (const char *const *names, char **values, char *description) {
+  if (names && values) {
+    while (*names) {
+      LogPrint(LOG_INFO, "%s: %s=%s", description, *names, *values);
+      ++names;
+      ++values;
+    }
+  }
+}
