@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -27,6 +27,40 @@
 #include "system.h"
 #include "sys_linux.h"
 
+#ifdef HAVE_LINUX_INPUT_H
+#include <linux/input.h>
+
+static int
+hasInputEvent (int device, uint16_t type, uint16_t code, uint16_t max) {
+  BITMASK(mask, max+1, long);
+
+  if (ioctl(device, EVIOCGBIT(type, max), mask) != -1)
+    if (BITMASK_TEST(mask, code))
+      return 1;
+
+  return 0;
+}
+
+static int
+writeInputEvent (const struct input_event *event) {
+  int device = getUinputDevice();
+
+  if (device != -1) {
+    if (write(device, event, sizeof(*event)) != -1) {
+      return 1;
+    } else {
+      LogError("write(struct input_event)");
+    }
+  }
+
+  return 0;
+}
+#endif /* HAVE_LINUX_INPUT_H */
+
+#ifdef HAVE_LINUX_UINPUT_H
+#include <linux/uinput.h>
+#endif /* HAVE_LINUX_UINPUT_H */
+
 char *
 getProgramPath (void) {
   char *path = NULL;
@@ -170,6 +204,8 @@ endBeep (void) {
 
 #include "sys_ports_glibc.h"
 
+#include "sys_kbd_linux.h"
+
 int
 installKernelModule (const char *name, int *status) {
   if (status && *status) return *status == 2;
@@ -279,14 +315,6 @@ openCharacterDevice (const char *name, int flags, int major, int minor) {
   return descriptor;
 }
 
-#ifdef HAVE_LINUX_INPUT_H
-#include <linux/input.h>
-#endif /* HAVE_LINUX_INPUT_H */
-
-#ifdef HAVE_LINUX_UINPUT_H
-#include <linux/uinput.h>
-#endif /* HAVE_LINUX_UINPUT_H */
-
 int
 getUinputDevice (void) {
   static int uinputDevice = -1;
@@ -351,32 +379,27 @@ getUinputDevice (void) {
   return uinputDevice;
 }
 
-static BITMASK(pressedKeys, KEY_MAX+1);
+static BITMASK(pressedKeys, KEY_MAX+1, char);
 
 int
 writeKeyEvent (int key, int press) {
-  int device = getUinputDevice();
-
-  if (device != -1) {
 #ifdef HAVE_LINUX_INPUT_H
-    struct input_event event;
+  struct input_event event;
 
-    event.type = EV_KEY;
-    event.code = key;
-    event.value = press;
+  event.type = EV_KEY;
+  event.code = key;
+  event.value = press;
 
-    if (write(device, &event, sizeof(event)) != -1) {
-      if (press) {
-        BITMASK_SET(pressedKeys, key);
-      } else {
-        BITMASK_CLEAR(pressedKeys, key);
-      }
-      return 1;
+  if (writeInputEvent(&event)) {
+    if (press) {
+      BITMASK_SET(pressedKeys, key);
+    } else {
+      BITMASK_CLEAR(pressedKeys, key);
     }
 
-    LogError("write(struct input_event)");
-#endif /* HAVE_LINUX_INPUT_H */
+    return 1;
   }
+#endif /* HAVE_LINUX_INPUT_H */
 
   return 0;
 }
