@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -22,14 +22,24 @@
 extern "C" {
 #endif /* __cplusplus */
 
+
 extern char *getProgramPath (void);
 
+
 extern char *getBootParameters (const char *name);
 
+
+extern int executeHostCommand (const char *const *arguments);
+
+
+extern int mountFileSystem (const char *path, const char *reference, const char *type);
+
+
 extern void *loadSharedObject (const char *path);
 extern void unloadSharedObject (void *object);
 extern int findSharedSymbol (void *object, const char *symbol, void *pointerAddress);
 
+
 extern int canBeep (void);
 extern int synchronousBeep (unsigned short frequency, unsigned short milliseconds);
 extern int asynchronousBeep (unsigned short frequency, unsigned short milliseconds);
@@ -37,7 +47,9 @@ extern int startBeep (unsigned short frequency);
 extern int stopBeep (void);
 extern void endBeep (void);
 
+
 typedef struct PcmDeviceStruct PcmDevice;
+
 typedef enum {
   PCM_FMT_U8,
   PCM_FMT_S8,
@@ -46,8 +58,10 @@ typedef enum {
   PCM_FMT_U16L,
   PCM_FMT_S16L,
   PCM_FMT_ULAW,
+  PCM_FMT_ALAW,
   PCM_FMT_UNKNOWN
 } PcmAmplitudeFormat;
+
 extern PcmDevice *openPcmDevice (int errorLevel, const char *device);
 extern void closePcmDevice (PcmDevice *pcm);
 extern int writePcmData (PcmDevice *pcm, const unsigned char *buffer, int count);
@@ -62,8 +76,10 @@ extern void forcePcmOutput (PcmDevice *pcm);
 extern void awaitPcmOutput (PcmDevice *pcm);
 extern void cancelPcmOutput (PcmDevice *pcm);
 
+
 typedef struct MidiDeviceStruct MidiDevice;
 typedef void (*MidiBufferFlusher) (unsigned char *buffer, int count);
+
 extern MidiDevice *openMidiDevice (int errorLevel, const char *device);
 extern void closeMidiDevice (MidiDevice *midi);
 extern int flushMidiDevice (MidiDevice *midi);
@@ -74,11 +90,18 @@ extern int startMidiNote (MidiDevice *midi, unsigned char channel, unsigned char
 extern int stopMidiNote (MidiDevice *midi, unsigned char channel);
 extern int insertMidiWait (MidiDevice *midi, int duration);
 
+
 extern int enablePorts (int errorLevel, unsigned short int base, unsigned short int count);
 extern int disablePorts (unsigned short int base, unsigned short int count);
 extern unsigned char readPort1 (unsigned short int port);
 extern void writePort1 (unsigned short int port, unsigned char value);
 
+
+#include "keyboard.h"
+
+extern int startKeyboardMonitor (const KeyboardProperties *keyboardProperties);
+
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
