@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -25,6 +25,7 @@
 #include <stdarg.h>
 
 #include "misc.h"
+#include "charset.h"
 #include "tbl.h"
 #include "tbl_internal.h"
 
@@ -224,6 +225,25 @@ loadTranslationTable (const char *path, FILE *file, TranslationTable table, int
   return tblLoad_Native(path, file, table, options);
 }
 
+unsigned char
+convertWcharToDots (TranslationTable table, wchar_t character) {
+  const wchar_t cellMask = 0XFF;
+
+  switch (character & ~cellMask) {
+    case BRL_UC_ROW:
+      return character & cellMask;
+
+    case 0XF000:
+      return table[character & cellMask];
+
+    default: {
+      int byte = convertWcharToChar(character);
+      if (byte == EOF) byte = '?';
+      return table[byte];
+    }
+  }
+}
+
 void
 reverseTranslationTable (TranslationTable from, TranslationTable to) {
   int byte;
