@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -21,7 +21,6 @@
 #include <string.h>
 #include <errno.h>
 #include <fcntl.h>
-#include <wchar.h>
 #include <pthread.h>
 #include <limits.h>
 #include <langinfo.h>
@@ -600,32 +599,24 @@ describe_AtSpiScreen (ScreenDescription *description) {
 }
 
 static int
-read_AtSpiScreen (ScreenBox box, unsigned char *buffer, ScreenCharacterProperty property) {
+readCharacters_AtSpiScreen (const ScreenBox *box, ScreenCharacter *buffer) {
   long x,y;
-  int c;
-  if (box.height<0 || box.width<0) return 0;
-  if (property == SCR_ATTRIB) {
-    memset(buffer,0x07,box.height*box.width);
-    return 1;
-  }
-  memset(buffer,' ',box.height*box.width);
+  if (!validateScreenBox(box, curNumCols, curNumRows)) return 0;
+  clearScreenCharacters(buffer,box->height*box->width);
   pthread_mutex_lock(&updateMutex);
   if (!curTerm) {
-    setScreenMessage(&box, buffer, property, nonatspi);
+    setScreenMessage(box, buffer, nonatspi);
     goto out;
   }
-  if (box.top+box.height>curNumRows)
-    box.height = curNumRows-box.top;
-  if (box.top<curNumRows && box.left<curNumCols)
-    for (y=0; y<box.height; y++)
-      if (curRowLengths[box.top+y])
-	for (x=0; x<box.width; x++) {
-	  if (box.left+x<curRowLengths[box.top+y] - (curRows[box.top+y][curRowLengths[box.top+y]-1]=='\n')) {
-	    if ((c = convertWcharToChar(curRows[box.top+y][box.left+x])) == EOF)
-	      c = '?';
-	    buffer[y*box.width+x] = c;
-	  }
-	}
+  for (y=0; y<box->height; y++) {
+    if (curRowLengths[box->top+y]) {
+      for (x=0; x<box->width; x++) {
+        if (box->left+x<curRowLengths[box->top+y] - (curRows[box->top+y][curRowLengths[box->top+y]-1]=='\n')) {
+          buffer[y*box->width+x].text = curRows[box->top+y][box->left+x];
+        }
+      }
+    }
+  }
 out:
   pthread_mutex_unlock(&updateMutex);
   return 1;
@@ -636,27 +627,10 @@ insertKey_AtSpiScreen (ScreenKey key) {
   long keysym;
   int modMeta=0, modControl=0;
 
-  if (key < SCR_KEY_ENTER) {
-    wint_t wc;
-    if (key & SCR_KEY_MOD_META) {
-      key &= ~SCR_KEY_MOD_META;
-      modMeta = 1;
-    }
+  setKeyModifiers(&key, SCR_KEY_CONTROL);
 
-    if (!(key & 0xE0)) {
-      key |= 0x40;
-      modControl = 1;
-    }
-
-    wc = convertCharToWchar(key);
-    if (wc == WEOF)
-      keysym = key; /* let's hope this is more or less correct */
-    else if (wc < 0x100)
-      keysym = wc; /* latin1 character */
-    else
-      keysym = 0x1000000 | wc;
-  } else {
-    switch (key) {
+  if (isSpecialKey(key)) {
+    switch (key & SCR_KEY_CHAR_MASK) {
       case SCR_KEY_ENTER:         keysym = XK_KP_Enter;  break;
       case SCR_KEY_TAB:           keysym = XK_Tab;       break;
       case SCR_KEY_BACKSPACE:     keysym = XK_BackSpace; break;
@@ -708,6 +682,24 @@ insertKey_AtSpiScreen (ScreenKey key) {
       case SCR_KEY_FUNCTION + 34: keysym = XK_F35;       break;
       default: LogPrint(LOG_WARNING, "key not insertable: %04X", key); return 0;
     }
+  } else {
+    wchar_t wc;
+
+    if (key & SCR_KEY_ALT_LEFT) {
+      key &= ~SCR_KEY_ALT_LEFT;
+      modMeta = 1;
+    }
+
+    if (key & SCR_KEY_CONTROL) {
+      key &= ~SCR_KEY_CONTROL;
+      modControl = 1;
+    }
+
+    wc = key & SCR_KEY_CHAR_MASK;
+    if (wc < 0x100)
+      keysym = wc; /* latin1 character */
+    else
+      keysym = 0x1000000 | wc;
   }
   LogPrint(LOG_DEBUG, "inserting key: %04X -> %s%s%ld",
            key,
@@ -758,7 +750,7 @@ static void
 scr_initialize (MainScreen *main) {
   initializeRealScreen(main);
   main->base.describe = describe_AtSpiScreen;
-  main->base.read = read_AtSpiScreen;
+  main->base.readCharacters = readCharacters_AtSpiScreen;
   main->base.insertKey = insertKey_AtSpiScreen;
   main->base.selectVirtualTerminal = selectVirtualTerminal_AtSpiScreen;
   main->base.switchVirtualTerminal = switchVirtualTerminal_AtSpiScreen;
