@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -39,6 +39,7 @@ static int shmFileDescriptor = -1;
 
 #include "misc.h"
 #include "system.h"
+#include "charset.h"
 
 #include "scr_driver.h"
 #include "screen.h"
@@ -173,16 +174,26 @@ describe_ScreenScreen (ScreenDescription *description) {
 }
 
 static int
-read_ScreenScreen (ScreenBox box, unsigned char *buffer, ScreenCharacterProperty property) {
+readCharacters_ScreenScreen (const ScreenBox *box, ScreenCharacter *buffer) {
   ScreenDescription description;                 /* screen statistics */
   describe_ScreenScreen(&description);
-  if (validateScreenBox(&box, description.cols, description.rows)) {
-    off_t start = 4 + (((property == SCR_TEXT)? 0: 1) * description.cols * description.rows) + (box.top * description.cols) + box.left;
+  if (validateScreenBox(box, description.cols, description.rows)) {
+    ScreenCharacter *character = buffer;
+    unsigned char *text = shmAddress + 4 + (box->top * description.cols) + box->left;
+    unsigned char *attributes = text + (description.cols * description.rows);
+    size_t increment = description.cols - box->width;
     int row;
-    for (row=0; row<box.height; row++) {
-      memcpy(buffer + (row * box.width),
-             shmAddress + start + (row * description.cols),
-             box.width);
+    for (row=0; row<box->height; row++) {
+      int column;
+      for (column=0; column<box->width; column++) {
+        wint_t wc = convertCharToWchar(*text++);
+        if (wc == WEOF) wc = L'?';
+        character->text = wc;
+        character->attributes = *attributes++;
+        character++;
+      }
+      text += increment;
+      attributes += increment;
     }
     return 1;
   }
@@ -191,20 +202,18 @@ read_ScreenScreen (ScreenBox box, unsigned char *buffer, ScreenCharacterProperty
 
 static int
 insertKey_ScreenScreen (ScreenKey key) {
-  unsigned char flags = getAuxiliaryData()[1];
+  const unsigned char flags = getAuxiliaryData()[1];
+  wchar_t character = key & SCR_KEY_CHAR_MASK;
   char buffer[3];
   char *sequence;
 
-  LogPrint(LOG_DEBUG, "insert key: %4.4X", key);
-  if (key < SCR_KEY_ENTER) {
-    sequence = buffer + sizeof(buffer);
-    *--sequence = 0;
-    *--sequence = key & 0XFF;
-    if (key & SCR_KEY_MOD_META) *--sequence = 0X1B;
-  } else {
+  LogPrint(LOG_DEBUG, "insert key: %04X", key);
+  setKeyModifiers(&key, 0);
+
+  if (isSpecialKey(key)) {
 #define KEY(key,string) case (key): sequence = (string); break
 #define CURSOR_KEY(key,string1,string2) KEY((key), ((flags & 0X01)? (string1): (string2)))
-    switch (key) {
+    switch (character) {
       KEY(SCR_KEY_ENTER, "\r");
       KEY(SCR_KEY_TAB, "\t");
       KEY(SCR_KEY_BACKSPACE, "\x7f");
@@ -243,11 +252,22 @@ insertKey_ScreenScreen (ScreenKey key) {
       KEY(SCR_KEY_FUNCTION+19, "\x1b[34~");
 
       default:
-        LogPrint(LOG_WARNING, "unsuported key: %4.4X", key);
+        LogPrint(LOG_WARNING, "unsuported key: %04X", key);
         return 0;
     }
 #undef CURSOR_KEY
 #undef KEY
+  } else {
+    int byte = convertWcharToChar(character);
+
+    if (byte == EOF) {
+      LogPrint(LOG_WARNING, "character not supported in local character set: 0X%04X", key);
+    }
+
+    sequence = buffer + sizeof(buffer);
+    *--sequence = 0;
+    *--sequence = byte;
+    if (key & SCR_KEY_ALT_LEFT) *--sequence = 0X1B;
   }
 
   return doScreenCommand("stuff", sequence, NULL);
@@ -277,7 +297,7 @@ scr_initialize (MainScreen *main) {
   initializeRealScreen(main);
   main->base.currentVirtualTerminal = currentVirtualTerminal_ScreenScreen;
   main->base.describe = describe_ScreenScreen;
-  main->base.read = read_ScreenScreen;
+  main->base.readCharacters = readCharacters_ScreenScreen;
   main->base.insertKey = insertKey_ScreenScreen;
   main->construct = construct_ScreenScreen;
   main->destruct = destruct_ScreenScreen;
