@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -55,14 +55,14 @@ static unsigned char latin2cp437[128] =
    176, 151, 163, 150, 129, 178, 254, 152};
 
 static int
-spk_construct (char **parameters)
+spk_construct (SpeechSynthesizer *spk, char **parameters)
 {
   return 1;
 }
 
 
 static void
-spk_say (const unsigned char *buffer, int len)
+spk_say (SpeechSynthesizer *spk, const unsigned char *buffer, size_t len, size_t count, const unsigned char *attributes)
 {
   unsigned char *pre_speech = (unsigned char *)PRE_SPEECH;
   unsigned char *post_speech = (unsigned char *)POST_SPEECH;
@@ -98,7 +98,7 @@ spk_say (const unsigned char *buffer, int len)
 
 
 static void
-spk_mute (void)
+spk_mute (SpeechSynthesizer *spk)
 {
   unsigned char *mute_seq = (unsigned char *)MUTE_SEQ;
   serialWriteData (MB_serialDevice, mute_seq+1, mute_seq[0]);
@@ -106,6 +106,6 @@ spk_mute (void)
 
 
 static void
-spk_destruct (void)
+spk_destruct (SpeechSynthesizer *spk)
 {
 }
