@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -42,6 +42,10 @@ typedef enum {
 #define SPK_HAVE_RATE
 #define SPK_HAVE_VOLUME
 #include "spk_driver.h"
+
+struct theta_sfx_block_ops { /* used but not defined in header */
+  char dummy;
+};
 #include <theta.h>
 
 static cst_voice *voice = NULL;
@@ -79,7 +83,7 @@ loadVoice (theta_voice_desc *descriptor) {
 }
 
 static int
-spk_construct (char **parameters) {
+spk_construct (SpeechSynthesizer *spk, char **parameters) {
   theta_voice_search criteria;
   memset(&criteria, 0, sizeof(criteria));
   initializeTheta();
@@ -156,8 +160,8 @@ spk_construct (char **parameters) {
 }
 
 static void
-spk_destruct (void) {
-  spk_mute();
+spk_destruct (SpeechSynthesizer *spk) {
+  spk_mute(spk);
 
   if (voice) {
     theta_unload_voice(voice);
@@ -177,7 +181,7 @@ doChild (void) {
 }
 
 static void
-spk_say (const unsigned char *buffer, int length) {
+spk_say (SpeechSynthesizer *spk, const unsigned char *buffer, size_t length, size_t count, const unsigned char *attributes) {
   if (voice) {
     if (child != -1) goto ready;
 
@@ -204,7 +208,7 @@ spk_say (const unsigned char *buffer, int length) {
 }
 
 static void
-spk_mute (void) {
+spk_mute (SpeechSynthesizer *spk) {
   if (child != -1) {
     close(*pipeInput);
     close(*pipeOutput);
@@ -216,11 +220,11 @@ spk_mute (void) {
 }
 
 static void
-spk_rate (float setting) {
-  theta_set_rate_stretch(voice, 1.0/setting, NULL);
+spk_rate (SpeechSynthesizer *spk, unsigned char setting) {
+  theta_set_rate_stretch(voice, 1.0/getFloatSpeechRate(setting), NULL);
 }
 
 static void
-spk_volume (float setting) {
-  theta_set_rescale(voice, setting, NULL);
+spk_volume (SpeechSynthesizer *spk, unsigned char setting) {
+  theta_set_rescale(voice, getFloatSpeechVolume(setting), NULL);
 }
